@@ -12,7 +12,7 @@
 #include <dirent.h>
 #include <unistd.h>
 #define direct dirent
-#else
+#elif !defined(_MSC_VER)
 #include <sys/dir.h>
 #endif
 
@@ -20,20 +20,30 @@
 #include <errno.h>
 #include <limits.h>
 
+#ifndef XC_WIN32
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include <X11/Shell.h>
 #include <X11/Xutil.h>
 #include <X11/cursorfont.h>
-
+#endif
 #ifdef TCL_WRAPPER 
 #include <tk.h>
 #else
+#ifndef XC_WIN32
 #include "Xw/Xw.h"
 #include "Xw/WorkSpace.h"
 #include "Xw/TextEdit.h"
 #include "Xw/Toggle.h"
 #endif
+#endif
+
+#if defined(XC_WIN32) && defined(TCL_WRAPPER)
+#include <X11/Xlib.h>
+#include <X11/Xutil.h>
+#include <X11/cursorfont.h>
+#endif
+
 
 /*-------------------------------------------------------------------------*/
 /* Local includes							   */
@@ -78,6 +88,8 @@ GC	 hgc = NULL, sgc = NULL;
 char	 *cwdname = NULL;
 fileliststruct *files;
 
+#if !defined(XC_WIN32) || defined(TCL_WRAPPER)
+
 /*-------------------------------------------------------------------------*/
 /* Compare two filenames (for use by qsort())				   */
 /*-------------------------------------------------------------------------*/
@@ -109,6 +121,8 @@ void dragfilebox(xcWidget w, caddr_t clientdata, XMotionEvent *event)
 	   10 + FILECHARHEIGHT * (flcurrent
 	   - flstart), flcurwidth + 10, FILECHARHEIGHT);
 
+   if (files == NULL) return;
+
    twidth = XTextWidth(appdata.filefont, files[filenum].filename,
 	    strlen(files[filenum].filename));
    XDrawRectangle(dpy, lwin, areastruct.gc, 5,
@@ -170,6 +184,8 @@ void endfiletrack(xcWidget w, caddr_t clientdata, XCrossingEvent *event)
    XSetFunction(dpy, areastruct.gc, areastruct.gctype);
 }
 
+#endif
+
 /*----------------------------------------------------------------------*/
 /* Read a crash file to find the name of the original file.		*/
 /*----------------------------------------------------------------------*/
@@ -200,6 +216,7 @@ char *getcrashfilename()
 	    break;
 	 }
       }
+      fclose(fi);
    }
    return retstr;
 }
@@ -228,7 +245,9 @@ void findcrashfiles()
    DIR *cwd;
    struct direct *dp;
    struct stat sbuf;
+#ifndef _MSC_VER
    uid_t userid = getuid();
+#endif
    time_t recent = 0;
 
    cwd = opendir(xobjs.tempdir);
@@ -237,7 +256,11 @@ void findcrashfiles()
    while ((dp = readdir(cwd)) != NULL) {
       sprintf(_STR, "%s/%s", xobjs.tempdir, dp->d_name);
       if (!strncmp(_STR + strlen(xobjs.tempdir) + 1, "XC", 2)) {
-         if ((!stat(_STR, &sbuf)) && (sbuf.st_uid == userid)) {
+         if ((!stat(_STR, &sbuf))
+#ifndef _MSC_VER
+	     && (sbuf.st_uid == userid)
+#endif
+	     ) {
 	    if ((recent == 0) || (sbuf.st_ctime > recent)) {
 	       recent = sbuf.st_ctime;
 	       strcpy(_STR2, _STR);
@@ -272,6 +295,8 @@ void findcrashfiles()
 /* Match a filename extension against the file filter string.		*/
 /*----------------------------------------------------------------------*/
 
+#if !defined(XC_WIN32) || defined(TCL_WRAPPER)
+
 Boolean match_filter(char *fname, char *filter)
 {
    char *dotptr = strrchr(fname, '.');
@@ -290,7 +315,9 @@ Boolean match_filter(char *fname, char *filter)
 void listfiles(xcWidget w, popupstruct *okaystruct, caddr_t calldata)
 {
    XGCValues	values;
+#ifndef TCL_WRAPPER
    Arg wargs[2];
+#endif
    DIR *cwd;
    char *filter;
    Window lwin = xcWindow(w);
@@ -443,14 +470,14 @@ void newfilelist(xcWidget w, popupstruct *okaystruct)
 #ifdef TCL_WRAPPER
    int bval;
    int result;
-   char *cstr = Tcl_GetVar(xcinterp, "filter", TCL_NAMESPACE_ONLY);
+   char *cstr = (char *)Tcl_GetVar2(xcinterp, "XCOps", "filter", TCL_NAMESPACE_ONLY);
    if (cstr == NULL) {
-      Wprintf("Error: No variable $filter in Tcl!");
+      Wprintf("Error: No variable $XCOps(filter) in Tcl!");
       return;
    }
    result = Tcl_GetBoolean(xcinterp, cstr, &bval);
    if (result != TCL_OK) {
-      Wprintf("Error: Bad variable $filter in Tcl!");
+      Wprintf("Error: Bad variable $XCOps(filter) in Tcl!");
       return;  
    }
    areastruct.filefilter = bval;
@@ -483,7 +510,9 @@ void newfilelist(xcWidget w, popupstruct *okaystruct)
 
 void fileselect(xcWidget w, popupstruct *okaystruct, XButtonEvent *event)
 {
+#ifndef TCL_WRAPPER
    Arg wargs[2];
+#endif
    Window lwin = xcWindow(w);
    Dimension textwidth, textheight;
    short filenum, n = 0;
@@ -501,6 +530,8 @@ void fileselect(xcWidget w, popupstruct *okaystruct, XButtonEvent *event)
    XtGetValues(w, wargs, n);
 #endif
 
+   if (files == NULL) return;	/* shouldn't happen */
+
    /* third mouse button cancels selection and reverts buffer to cwd name */
 
    if (event->button == Button3) {
@@ -603,7 +634,9 @@ void fileselect(xcWidget w, popupstruct *okaystruct, XButtonEvent *event)
 
 void showlscroll(xcWidget w, caddr_t clientdata, caddr_t calldata)
 {
+#ifndef TCL_WRAPPER
    Arg wargs[2];
+#endif
    Window swin = xcWindow(w);
    Dimension swidth, sheight;
    int pstart, pheight, finscr;
@@ -640,7 +673,9 @@ void showlscroll(xcWidget w, caddr_t clientdata, caddr_t calldata)
 
 void draglscroll(xcWidget w, popupstruct *okaystruct, XButtonEvent *event)
 {
+#ifndef TCL_WRAPPER
    Arg wargs[1];
+#endif
    Dimension sheight;
    int phheight, finscr, flsave = flstart;
    short n = 0;
@@ -784,6 +819,8 @@ void genfilelist(xcWidget parent, popupstruct *okaystruct, Dimension width)
 
 #endif 	/* TCL_WRAPPER */
 
+#endif
+
 /*-------------------------------------------------------------------------*/
 /* Look for a directory name in a string and update cwdname accordingly	   */
 /*-------------------------------------------------------------------------*/
@@ -791,11 +828,13 @@ void genfilelist(xcWidget parent, popupstruct *okaystruct, Dimension width)
 int lookdirectory(char *lstring)
 {
    int slen;
+   DIR *cwd = NULL;
 
    xc_tilde_expand(lstring);
    slen = strlen(lstring);
 
-   if (lstring[slen - 1] == '/' || (opendir(lstring) != NULL)) {
+   if (lstring[slen - 1] == '/' || ((cwd=opendir(lstring)) != NULL)) {
+      if (cwd) closedir(cwd);
       if (lstring[slen - 1] != '/') strcat(lstring, "/");
       cwdname = (char *)realloc(cwdname, (slen + 2) * sizeof(char));
       strcpy(cwdname, lstring);
