@@ -8,14 +8,18 @@
 #include <ctype.h>
 #include <string.h>
 
+#ifndef XC_WIN32
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
+#endif
 
 #ifdef TCL_WRAPPER 
 #include <tk.h>
 #else
+#ifndef XC_WIN32
 #include "Xw/Xw.h"
 #endif
+#endif
 
 /*-------------------------------------------------------------------------*/
 /* Local includes                                                          */
@@ -53,7 +57,7 @@ extern Tcl_Interp *xcinterp;
 FILE *findfontfile(char *fontname)
 {
    int i;
-   char tempname[100];
+   char tempname[256];
    char *tmp_s = getenv((const char *)"XCIRCUIT_LIB_DIR");
    FILE *fd;
 
@@ -158,7 +162,7 @@ FILE *findfontfile(char *fontname)
 			fontname);
 	       Fprintf(stderr, "No fonts exist for a subsitution.  Make sure "
 			"fonts are installed or that\nenvironment variable "
-			"XCIRUCIT_LIB_DIR points to a directory of valid fonts.\n");
+			"XCIRCUIT_LIB_DIR points to a directory of valid fonts.\n");
 	    }
             return (FILE *)NULL;
          }
@@ -172,6 +176,7 @@ FILE *findfontfile(char *fontname)
 /* Return 1 if successful, 0 if font is already present, and -1 if any	*/
 /* other error occurred.						*/
 /*----------------------------------------------------------------------*/
+extern Clientdata areastruct;
 
 int loadfontfile(char *fontname)
 {
@@ -213,6 +218,7 @@ int loadfontfile(char *fontname)
          for (i = 0; i < fontcount; i++) {
             if (!strcmp(fonts[i].psname, tempname)) {
 	       /* Fprintf(stdout, "Font already loaded.\n"); */
+	       fclose(fd);
 	       return 0;
 	    }
 	 }
@@ -326,15 +332,16 @@ int loadfontfile(char *fontname)
 	 /* ISO-LatinX encodings where X=1 to 6 */
 
 	 if (strstr(tempname, "ISO") != NULL) {
-	    char estr[30];
+	    char estr[38];
 	    for (i = 0; i < 6; i++) {
 	       if (strchr(tempname, '1' + (char)i) != NULL) {
 	          flags |= ((i + 2) << 7);
 #ifdef TCL_WRAPPER
-		  sprintf(estr, "xcircuit::newencodingbutton ISOLatin%d", i + 1);
+		  snprintf(estr, sizeof(estr),
+			"xcircuit::newencodingbutton ISOLatin%d", i + 1);
 		  Tcl_Eval(xcinterp, estr);
 #else
-		  sprintf(estr, "ISO-Latin%d", i + 1);
+		  snprintf(estr, sizeof(estr), "ISO-Latin%d", i + 1);
 	          makenewencodingbutton(estr, (char)(i + 2));
 #endif
 		  break;
@@ -403,5 +410,6 @@ int loadfontfile(char *fontname)
 	    makenewfontbutton();
       }
    }
+   fclose(fd);
    return 1;
 }
