@@ -7,8 +7,10 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#ifndef _MSC_VER
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
+#endif
 
 /*------------------------------------------------------------------------*/
 /* Local includes                                                         */
@@ -47,8 +49,6 @@ extern int *appcolors;
 /*----------------------------------------------*/
 #ifdef LGF
 
-Boolean lgfisloaded = False;
-
 /*----------------------------------------------*/
 /* loadlgf: Load an analog LGF file 		*/
 /*	mode = 1: import into current page;	*/
@@ -91,12 +91,11 @@ void loadlgf(int mode)
 
    /* Make sure that LGF object library has been loaded by loading it now. */
 
-   if (!lgfisloaded) {
+   if (check_library(LGF_LIB) < 0) {
       int ilib;
       strcpy(_STR, LGF_LIB);
-      ilib = createlibrary();
+      ilib = createlibrary(FALSE);
       loadlibrary(ilib);
-      lgfisloaded = True;
    }
    
    /* Read header information */
@@ -248,9 +247,7 @@ void loadlgf(int mode)
 		  strptr->type = TEXT_STRING;
 		  strptr->data.string = (char *)malloc(1 + strlen(tstrp));
 		  strcpy(strptr->data.string, tstrp);
-#ifdef SCHEMA
 		  (*newlabel)->pin = NORMAL;
-#endif
 	       }
             }}
 	    break;
@@ -280,7 +277,6 @@ void loadlgf(int mode)
 	       (*newwire)->width = 1.0;
 	       (*newwire)->style = UNCLOSED;
 	       (*newwire)->color = DEFAULTCOLOR;
-	       (*newwire)->num_params = 0;
 	       (*newwire)->passed = NULL;
 	       tmppnts = (*newwire)->points;
 	       tmppnts->x = xmat(pvalx);
@@ -331,7 +327,6 @@ void loadlgf(int mode)
 	       (*newpoly)->width = 1.0;
 	       (*newpoly)->number = 4;
                (*newpoly)->points = (pointlist) malloc(4 * sizeof(XPoint));
-	       (*newpoly)->num_params = 0;
 	       (*newpoly)->passed = NULL;
 
                newpoints = (*newpoly)->points;
@@ -400,7 +395,6 @@ void loadlgf(int mode)
 		  (*newinst)->scale = 1.0;
 		  (*newinst)->color = DEFAULTCOLOR;
 		  (*newinst)->params = NULL;
-	          (*newinst)->num_params = 0;
 	          (*newinst)->passed = NULL;
 
 		  if (pvalx2 & 0x01) pvalx2 ^= 0x02;
@@ -434,12 +428,10 @@ void loadlgf(int mode)
 			   labeldefaults(*newlabel, False, (*newinst)->position.x,
 				(*newinst)->position.y);
 			   (*newlabel)->color = DEFAULTCOLOR;
-#ifdef SCHEMA
 		  	   if (areastruct.schemon){
 			      (*newlabel)->pin = LOCAL;
 			      (*newlabel)->color = LOCALPINCOLOR;
 		  	   }
-#endif
 			   if (!strcmp(tmpstring, "TO"))
 			      (*newlabel)->position.x += ((flip) ? 48 : -48);
 			   else
@@ -517,7 +509,7 @@ void loadlgf(int mode)
 
    for (iolabel = topobject->plist; iolabel < topobject->plist +
 	   topobject->parts; iolabel++)
-      if ((*iolabel)->type == LABEL) {
+      if (IS_LABEL(*iolabel)) {
          if (TOLABEL(iolabel)->rotation == 500) {
 	    genericptr *tmplabel;
 
