@@ -8,16 +8,20 @@
 /*-------------------------------------------------------------------------*/
 
 #include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
+#include <stdlib.h>
 
+#ifndef XC_WIN32
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
+#endif
 #ifdef TCL_WRAPPER 
 #include <tk.h>
 #else
+#ifndef XC_WIN32
 #include "Xw/Xw.h"
 #endif
+#endif
 
 #include <math.h>
 
@@ -47,6 +51,10 @@ extern char _STR[150];
 extern short fontcount;
 extern fontinfo *fonts;
 
+/*------------------------------------------------------*/
+/* Check if a library exists				*/
+/*-------------------------------------------------------*/
+
 /*---------------------------------------------------------*/
 /* Find the Helvetica font for use in labeling the objects */
 /*---------------------------------------------------------*/
@@ -87,7 +95,7 @@ void catreturn()
 {
    /* Pop the object being edited from the push stack. */
 
-   popobject(NULL, NULL, NULL);
+   popobject(NULL, (pointertype)1, NULL);
 }
 
 /*------------------------------------------------------*/
@@ -98,14 +106,14 @@ void catreturn()
 /*   page number of page to the right.  		*/
 /*------------------------------------------------------*/
 
-int pageposition(short libmode, XButtonEvent *event, int mode)
+int pageposition(short libmode, int x, int y, int mode)
 {
    int xin, yin, bpage, pages;
    int gxsize, gysize, xdel, ydel;
 
    pages = (libmode == PAGELIB) ? xobjs.pages : xobjs.numlibs;
    computespacing(libmode, &gxsize, &gysize, &xdel, &ydel);
-   window_to_user(event->x, event->y, &areastruct.save);
+   window_to_user(x, y, &areastruct.save);
 
    if (mode == 0) {	/* On-page */
       if (areastruct.save.x >= 0 && areastruct.save.y <= 0) {
@@ -273,11 +281,27 @@ int is_library(objectptr thisobj)
    return -1;
 }
 
+/*------------------------------------------------------*/
+/* Check if a library (given by name) exists.  If it	*/
+/* does, return the library number.  If not, return -1	*/
+/*------------------------------------------------------*/
+
+int check_library(char *libname)
+{
+   int i;
+   
+   for (i = 0; i < xobjs.numlibs; i++)
+      if (!strcmp(xobjs.libtop[i + LIBRARY]->thisobject->name, libname))
+	 return i;
+
+   return -1;
+}
+
 /*------------------------------------------------------*/
 /* ButtonPress handler during page catalog viewing mode */
 /*------------------------------------------------------*/
 
-void pagecatbutton(XButtonEvent *event)
+void pagecat_op(int op, int x, int y)
 {
    int bpage;
    short mode;
@@ -287,10 +311,9 @@ void pagecatbutton(XButtonEvent *event)
    }
    if (mode == LIBRARY) return;  /* Something went wrong if this happens */
 
-   if (event->button == Button1 || event->button == Button2) {
-      if ((bpage = pageposition(mode, event, 0)) >= 0) {
+   if (op != XCF_Cancel) {
+      if ((bpage = pageposition(mode, x, y, 0)) >= 0) {
 		  
-#ifdef SCHEMA
 	 if (eventmode == ASSOC_MODE) {
 	    if (mode == PAGELIB) {
 	       /* using changepage() allows use of new page for schematic */
@@ -299,33 +322,31 @@ void pagecatbutton(XButtonEvent *event)
 	       schemassoc(topobject, areastruct.stack->thisinst->thisobject);
 	       /* pop back to calling (symbol) page */
 	       catreturn();
-	       eventmode = PRESS_MODE;
+	       eventmode = NORMAL_MODE;
 	    }
 	    else {
+	       areastruct.lastlibrary = bpage;
 	       startcatalog(NULL, (pointertype)(LIBRARY + bpage), NULL);
 	    }
 	    return;
          }
-	 else
-#endif
-	 if (event->button == Button1) {
+	 else if (op == XCF_Select) {
+	    if (mode == PAGELIB)    /* No such method for LIBLIB is defined. */
+	       select_add_element(OBJINST);
+	 }
+	 else if ((op == XCF_Library_Pop) || (op == XCF_Finish)) {
 
 	    /* like catreturn(), but don't actually go to the popped page */
-	    objectdeselect();
+	    unselect_all();
 	    eventmode = NORMAL_MODE;
 	    if (mode == PAGELIB) {
 	       newpage(bpage);
-      	       eventmode = PRESS_MODE;
 	    }
 	    else {
 	       startcatalog(NULL, (pointertype)(LIBRARY + bpage), NULL);
 	    }
 	    return;
 	 }
-	 else {  /* Button2 */
-	    if (mode == PAGELIB)    /* No such method for LIBLIB is defined. */
-	       objectselect(OBJECT);
-	 }
       }
    }
    else {
@@ -382,8 +403,8 @@ void pageinstpos(short mode, short tpage, objinstptr drawinst, int gxsize,
 objinstptr newpageinst(objectptr pageobj)
 {
    objinstptr newinst = (objinstptr) malloc(sizeof(objinst));
-   objectdefaults(newinst, pageobj, 0, 0);
-   newinst->type = OBJECT;
+   instancedefaults(newinst, pageobj, 0, 0);
+   newinst->type = OBJINST;
    newinst->color = DEFAULTCOLOR;
    return newinst;
 }
@@ -428,7 +449,7 @@ void composepagelib(short mode)
    /* from the list.							 */
 
    for (pgen = directory->plist; pgen < directory->plist + directory->parts; pgen++)
-      if ((*pgen)->type == OBJECT) *pgen = NULL;
+      if (IS_OBJINST(*pgen)) *pgen = NULL;
 
    reset(directory, NORMAL);
 
@@ -468,7 +489,6 @@ void composepagelib(short mode)
       (*drawbox)->width = 1.0;
       (*drawbox)->number = 4;
       (*drawbox)->points = (pointlist) malloc(4 * sizeof(XPoint));
-      (*drawbox)->num_params = 0;
       (*drawbox)->passed = NULL;
       pointptr = (*drawbox)->points;
       pointptr->x = (i % gxsize) * xdel + margin;
@@ -493,7 +513,6 @@ void composepagelib(short mode)
 	 (*pagelabel)->color = DEFAULTCOLOR;
 	 (*pagelabel)->scale = 0.75;
 	 (*pagelabel)->string->data.font = fval;
-         (*pagelabel)->num_params = 0;
          (*pagelabel)->passed = NULL;
 	 strptr = makesegment(&((*pagelabel)->string), NULL);
 	 strptr->type = TEXT_STRING;
@@ -532,7 +551,7 @@ void updatepagelib(short mode, short tpage)
 
    for (i = 0; i < libinst->parts; i++) {
       gelem = libinst->plist + i;
-      if ((*gelem)->type == OBJECT) {
+      if (IS_OBJINST(*gelem)) {
 	 pinst = TOOBJINST(gelem);
          if (pinst->thisobject == compobj) {
 	    /* recalculate scale and position of the object instance */
@@ -551,7 +570,7 @@ void updatepagelib(short mode, short tpage)
 /* Rearrange pages	*/
 /*----------------------*/
 
-void pagecatmove(XKeyEvent *event)
+void pagecatmove(int x, int y)
 {
    int bpage;
    objinstptr exchobj;
@@ -585,7 +604,7 @@ void pagecatmove(XKeyEvent *event)
 
    /* If one object selected; find place to put from cursor position */
 
-   else if ((bpage = pageposition(PAGELIB, (XButtonEvent *)event, 1)) >= 0) {
+   else if ((bpage = pageposition(PAGELIB, x, y, 1)) >= 0) {
       int k, epage;
       Pagedata *eptr;
 
@@ -615,7 +634,7 @@ void pagecatmove(XKeyEvent *event)
       }
    }
 
-   objectdeselect();
+   unselect_all();
    composelib(PAGELIB);
    drawarea(NULL, NULL, NULL);
 }
@@ -636,6 +655,9 @@ void composelib(short mode)
    short fval;
    short llx, lly, width, height;
 
+   int totalarea, targetwidth;
+   double scale;
+
    /* Also make composelib() a wrapper for composepagelib() */
    if ((mode > FONTLIB) && (mode < LIBRARY)) {
       composepagelib(mode);
@@ -647,18 +669,47 @@ void composelib(short mode)
    /* call reset(), we find the pointer to the instance and NULL it.	*/
    
    for (pgen = libpage->plist; pgen < libpage->plist + libpage->parts; pgen++)
-      if ((*pgen)->type == OBJECT) *pgen = NULL;
+      if (IS_OBJINST(*pgen)) *pgen = NULL;
 
    reset(libpage, NORMAL);
 
-   /* Return if library defines no objects */
+   /* Return if library defines no objects or virtual instances */
 
-   if (xobjs.userlibs[mode - LIBRARY].number == 0) return;
+   if (xobjs.userlibs[mode - LIBRARY].instlist == NULL) return;
 
    /* Find the Helvetica font for use in labeling the objects */
 
    fval = findhelvetica();
 
+   /* experimental:  attempt to produce a library with the same aspect  */
+   /* ratio as the drawing window.                                      */
+
+   totalarea = 0;
+   for (spec = xobjs.userlibs[mode - LIBRARY].instlist; spec != NULL;
+                spec = spec->next) {
+      libobj = spec->thisinst->thisobject;
+
+      /* "Hidden" objects are not drawn */
+      if (libobj->hidden == True) continue;
+
+      drawinst = spec->thisinst;
+      drawinst->position.x = 0;
+      drawinst->position.y = 0;
+
+      /* Get the bounding box of the instance in the page's coordinate system */
+      calcinstbbox((genericptr *)(&drawinst), &llx, &lly, &width, &height);
+      width -= llx;	/* convert urx to width */
+      height -= lly;	/* convert ury to height */
+      width += 30;	/* space padding */
+      height += 30;	/* height padding */
+      if (width < 200) width = 200;	/* minimum box width */
+      if (height < 220) height = 220;	/* minimum box height */
+      totalarea += (width * height);
+   }
+
+   scale = (double)totalarea / (double)(areastruct.width * areastruct.height);
+   targetwidth = (int)(sqrt(scale) * (double)areastruct.width);
+
    /* generate the list of object instances and their labels */
 
    for (spec = xobjs.userlibs[mode - LIBRARY].instlist; spec != NULL;
@@ -680,7 +731,8 @@ void composelib(short mode)
       /* Determine the area needed on the page to draw the object */
 
       nxpos = xpos + ((width > 170) ? width + 30 : 200);
-      if ((nxpos > (areastruct.width << 1)) && (xpos > 0)) {
+      /* if ((nxpos > (areastruct.width << 1)) && (xpos > 0)) { */
+      if ((nxpos > targetwidth) && (xpos > 0)) {
 	 nxpos -= xpos; 
 	 xpos = 0;
 	 ypos -= nypos;
@@ -756,7 +808,7 @@ short finddepend(objinstptr libobj, objectptr **compobjp)
 		     
          for (testobj = (*compobj)->plist; testobj < (*compobj)->plist
 	          + (*compobj)->parts; testobj++) {
-	    if ((*testobj)->type == OBJECT) {
+	    if (IS_OBJINST(*testobj)) {
 	       if (TOOBJINST(testobj)->thisobject == libobj->thisobject) return 2;
 	    }
 	 }
@@ -771,7 +823,7 @@ short finddepend(objinstptr libobj, objectptr **compobjp)
       *compobjp = compobj;
       for (testobj = (*compobj)->plist; testobj < (*compobj)->plist
 	       + (*compobj)->parts; testobj++) {
-	 if ((*testobj)->type == OBJECT) {
+	 if (IS_OBJINST(*testobj)) {
 	    if (TOOBJINST(testobj)->thisobject == libobj->thisobject) return 1;
 	 }
       }
@@ -802,7 +854,7 @@ void catvirtualcopy()
 	  + areastruct.selects; newselect++) {
       libobj = SELTOOBJINST(newselect);
 
-      if (libobj->thisobject->num_params == 0) {
+      if (libobj->thisobject->params == NULL) {
 	 Wprintf("Virtual copy allowed only on objects with paramters.");
 	 /* (because currently there's no way to adjust rotation/scale	*/
 	 /* of a non-parameterized instance from inside the library,	*/
@@ -906,24 +958,14 @@ void catdelete()
       }
       else { 		/* All's clear to delete.		     	   */
 
-         /* First remove any instances of this object in the delete buffer */
+         /* Clear the undo stack so that any references to this object	*/
+	 /* won't screw up the database (this could be kinder & gentler	*/
+	 /* by first checking if there are any references to the object	*/
+	 /* in the undo stack. . .					*/
 
-         for (compobj = xobjs.delbuffer.library; compobj != xobjs.delbuffer.library +
-                xobjs.delbuffer.number; compobj++) {
-	    for (testobj = (*compobj)->plist; testobj < (*compobj)->plist
-	          + (*compobj)->parts; testobj++) {
-	       if ((*testobj)->type == OBJECT) {
-	          if (TOOBJINST(testobj)->thisobject == libobj->thisobject) {
-		     free(TOOBJINST(testobj));
-		     for (tobj = testobj; tobj < (*compobj)->plist
-			   + (*compobj)->parts - 1; tobj++) {
-		        (*tobj) = (*(tobj + 1));
-		     }
-		     (*compobj)->parts--;
-		  }
-	       }
-	    }
-         }
+	 flush_undo_stack();
+
+	 /* Next, remove the object from the library page. */
 
 	 for (tlib = libpage; tlib < libpage + *libpobjs; tlib++)
 	    if ((*tlib) == libobj->thisobject) {
@@ -1043,7 +1085,7 @@ void linkedlistinsertafter(liblistptr *spec, int o1, int o2)
 /* Rearrange objects in the library			*/
 /*------------------------------------------------------*/
 
-void catmove(XKeyEvent *event)
+void catmove(int x, int y)
 {
    int i, j, s1, s2, ocentx, ocenty, rangex, rangey;
    liblistptr spec;
@@ -1052,7 +1094,7 @@ void catmove(XKeyEvent *event)
    /* make catmove() a wrapper for pagecatmove() */
 
    if ((i = is_library(topobject)) < 0) {
-      pagecatmove(event);
+      pagecatmove(x, y);
       return;
    }
 
@@ -1090,7 +1132,7 @@ void catmove(XKeyEvent *event)
    }
    else {	/* one object selected; find place to put from cursor position */
 
-      window_to_user(event->x, event->y, &areastruct.save);
+      window_to_user(x, y, &areastruct.save);
 
       s2 = -1;
       for (j = 0, spec = xobjs.userlibs[i].instlist; spec != NULL;
@@ -1118,7 +1160,7 @@ void catmove(XKeyEvent *event)
 	 else if (areastruct.save.y <=
 		xobjs.libtop[i + LIBRARY]->thisobject->bbox.lowerleft.y +
 		xobjs.libtop[i + LIBRARY]->thisobject->bbox.height) {
-	    objectdeselect();	
+	    unselect_all();	
 	    Wprintf("Could not find appropriate place to insert object");
 	    return;
 	 }
@@ -1128,7 +1170,7 @@ void catmove(XKeyEvent *event)
       linkedlistinsertafter(&(xobjs.userlibs[i].instlist), s1, s2);
    }
 
-   objectdeselect();
+   unselect_all();
    if ((i = is_library(topobject)) >= 0) composelib(LIBRARY + i);
 
    drawarea(NULL, NULL, NULL);
@@ -1143,6 +1185,7 @@ void copycat()
    short *newselect;
    objectptr *newobj, *curlib, oldobj;
    objinstptr libobj;
+   oparamptr ops, newops;
    int i, libnum = USERLIB - LIBRARY ;
 
    for (newselect = areastruct.selectlist; newselect < areastruct.selectlist
@@ -1174,41 +1217,45 @@ void copycat()
       (*newobj)->pcorner.x = oldobj->pcorner.x;
       (*newobj)->pcorner.y = oldobj->pcorner.y;
       (*newobj)->viewscale = oldobj->viewscale;
-#ifdef SCHEMA
       /* don't attach the same schematic. . . */
       (*newobj)->symschem = NULL;
+      /* don't copy highlights */
+      (*newobj)->highlight.netlist = NULL;
+      (*newobj)->highlight.thisinst = NULL;
 
       /* Copy the parameter structure */
-      (*newobj)->num_params = oldobj->num_params;
-      (*newobj)->params = (oparamptr *)malloc(oldobj->num_params *
-		sizeof(oparamptr));
-      for (i = 0; i < oldobj->num_params; i++) {
-	 (*newobj)->params[i] = (oparamptr) malloc(sizeof(oparam));
-	 (*newobj)->params[i]->type = oldobj->params[i]->type;
-	 switch (oldobj->params[i]->type) {
+      (*newobj)->params = NULL;
+      for (ops = oldobj->params; ops != NULL; ops = ops->next) {
+	 newops = (oparamptr)malloc(sizeof(oparam));
+	 newops->next = (*newobj)->params;
+	 newops->key = strdup(ops->key);
+	 (*newobj)->params = newops;
+	 newops->type = ops->type;
+	 newops->which = ops->which;
+	 switch (ops->type) {
 	    case XC_INT:
-	       (*newobj)->params[i]->parameter.ivalue =
-			oldobj->params[i]->parameter.ivalue;
+	       newops->parameter.ivalue = ops->parameter.ivalue;
 	       break;
 	    case XC_FLOAT:
-	       (*newobj)->params[i]->parameter.fvalue =
-			oldobj->params[i]->parameter.fvalue;
+	       newops->parameter.fvalue = ops->parameter.fvalue;
 	       break;
 	    case XC_STRING:
-	       (*newobj)->params[i]->parameter.string =
-			stringcopy(oldobj->params[i]->parameter.string);
+	       newops->parameter.string = stringcopy(ops->parameter.string);
+	       break;
+	    case XC_EXPR:
+	       newops->parameter.expr = strdup(ops->parameter.expr);
 	       break;
 	 }
       }
 
       (*newobj)->schemtype = oldobj->schemtype;
-      (*newobj)->netlist = NULL;
-      (*newobj)->portlist = NULL;
-      (*newobj)->calllist = NULL;
+      (*newobj)->netnames = NULL;
+      (*newobj)->ports = NULL;
+      (*newobj)->calls = NULL;
+      (*newobj)->polygons = NULL;
+      (*newobj)->labels = NULL;
       (*newobj)->valid = False;
       (*newobj)->traversed = False;
-      (*newobj)->devname = NULL;
-#endif
       (*newobj)->hidden = False;
 
       /* copy over all the elements of the original object */
@@ -1217,7 +1264,7 @@ void copycat()
       (*newobj)->plist = (genericptr *)malloc(sizeof(genericptr));
 
       for (i = 0; i < oldobj->parts; i++) {
-	 switch((*(oldobj->plist + i))->type) {
+	 switch(ELEMENTTYPE(*(oldobj->plist + i))) {
 	    case(PATH): {
 	       register pathptr *npath, cpath = TOPATH(oldobj->plist + i);
 	       register genericptr *gpart;
@@ -1227,11 +1274,10 @@ void copycat()
 	       (*npath)->parts = 0;
 	       (*npath)->plist = (genericptr *)malloc(cpath->parts *
 			sizeof(genericptr));
-               (*npath)->num_params = 0;
                (*npath)->passed = NULL;
 	       for (gpart = cpath->plist; gpart < cpath->plist + cpath->parts;
 			gpart++) {
-		  switch((*gpart)->type){
+		  switch(ELEMENTTYPE(*gpart)){
 		     case ARC: {
 			arcptr copyarc = TOARC(gpart);
 			arcptr *newarc;
@@ -1284,46 +1330,15 @@ void copycat()
 
 	       clabel = TOLABEL(oldobj->plist + i);
 	       NEW_LABEL(nlabel, (*newobj));
-	       (*nlabel)->rotation = clabel->rotation;
-	       (*nlabel)->color = clabel->color;
-	       (*nlabel)->position.x = clabel->position.x;
-	       (*nlabel)->position.y = clabel->position.y;
-	       (*nlabel)->scale = clabel->scale;
-	       (*nlabel)->justify = clabel->justify;
-	       (*nlabel)->string = stringcopy(clabel->string);
-	       (*nlabel)->num_params = 0;
-	       (*nlabel)->passed = NULL;
-#ifdef SCHEMA
-	       (*nlabel)->pin = clabel->pin;
-#endif
+	       labelcopy(*nlabel, clabel);
                } break;
       
-	    case(OBJECT): {
+	    case(OBJINST): {
 	       register objinstptr *ninst, cinst;
 
 	       cinst = TOOBJINST(oldobj->plist + i);
 	       NEW_OBJINST(ninst, (*newobj));
-	       (*ninst)->rotation = cinst->rotation;
-	       (*ninst)->color = cinst->color;
-	       (*ninst)->position.x = cinst->position.x;
-	       (*ninst)->position.y = cinst->position.y;
-	       (*ninst)->scale = cinst->scale;
-	       (*ninst)->thisobject = cinst->thisobject;
-	       (*ninst)->num_params = 0;
-	       (*ninst)->passed = NULL;
-
-	       /* Library instance's parameter list is always default (null) */
-	       (*ninst)->params = NULL;
-	       (*ninst)->bbox.lowerleft.x = cinst->thisobject->bbox.lowerleft.x;
-	       (*ninst)->bbox.lowerleft.y = cinst->thisobject->bbox.lowerleft.y;
-	       (*ninst)->bbox.width = cinst->thisobject->bbox.width;
-	       (*ninst)->bbox.height = cinst->thisobject->bbox.height;
-
-#ifdef SCHEMA
-	       (*ninst)->schembbox = NULL;
-
-	       /* here---deal with attached schematics */
-#endif
+	       instcopy(*ninst, cinst);
                } break;
          }
 	 (*newobj)->parts++;
@@ -1334,7 +1349,7 @@ void copycat()
    addtoinstlist(USERLIB - LIBRARY, *newobj, FALSE);
 
    composelib(USERLIB);
-   objectdeselect();
+   unselect_all();
 
    if (areastruct.topinstance == xobjs.libtop[USERLIB])
       drawarea(NULL, NULL, NULL);
@@ -1345,7 +1360,7 @@ void copycat()
 /* ButtonPress handler during normal catalog viewing mode */
 /*--------------------------------------------------------*/
 
-void catbutton(u_int mode, XButtonEvent *event)
+void catalog_op(int op, int x, int y)
 {
    short *newselect;
    objinstptr *newobject, *libobj;
@@ -1353,20 +1368,26 @@ void catbutton(u_int mode, XButtonEvent *event)
    short ocentx, ocenty, rangex, rangey, xdiff, ydiff, flag = 0;
    XPoint oldpos;
 
-   /* Make catbutton() a wrapper for pagecatbutton() */
+   /* Make catalog_op() a wrapper for pagecat_op() */
 
    if (is_library(topobject) < 0) {
-      pagecatbutton(event);
+      pagecat_op(op, x, y);
       return;
    }
 
-   if (event->button == Button1 || event->button == Button2) {
+   /* If XCF_Cancel was invoked, return without a selection. */
 
-      window_to_user(event->x, event->y, &areastruct.save);
+   if (op == XCF_Cancel) {
+      eventmode = NORMAL_MODE;
+      catreturn();
+   }
+   else {
+
+      window_to_user(x, y, &areastruct.save);
 	
       for (libobj = (objinstptr *)topobject->plist; libobj <
 		(objinstptr *)topobject->plist + topobject->parts; libobj++) {
-	 if ((*libobj)->type == OBJECT) {
+	 if (IS_OBJINST(*libobj)) {
 
  	    ocentx = (*libobj)->position.x + (*libobj)->bbox.lowerleft.x
 	        + ((*libobj)->bbox.width >> 1);
@@ -1384,7 +1405,6 @@ void catbutton(u_int mode, XButtonEvent *event)
 
 	       /* setup to move object around and draw the selected object */
 
-#ifdef SCHEMA
 	       if (eventmode == ASSOC_MODE) {
 
 	          /* revert to old page */
@@ -1395,11 +1415,9 @@ void catbutton(u_int mode, XButtonEvent *event)
 		  schemassoc(topobject, (*libobj)->thisobject); 
    	          setpage(TRUE);
 		  catreturn();
-		  eventmode = PRESS_MODE;
+		  eventmode = NORMAL_MODE;
 	       }
-	       else
-#endif
-               if (event->button == Button1) {
+	       else if ((op == XCF_Library_Pop) || (op == XCF_Library_Copy)) {
 
 	          /* revert to old page */
 
@@ -1410,11 +1428,11 @@ void catbutton(u_int mode, XButtonEvent *event)
 	          /* retrieve drawing window state and set position of object to
 	             be correct in reference to that window */
 
-	          snap(event->x, event->y, &oldpos);
+	          snap(x, y, &oldpos);
 
    	          setpage(FALSE);
 	    
-	          snap(event->x, event->y, &areastruct.save);
+	          snap(x, y, &areastruct.save);
 	          xdiff = areastruct.save.x - oldpos.x;
 	          ydiff = areastruct.save.y - oldpos.y;
 
@@ -1453,32 +1471,37 @@ void catbutton(u_int mode, XButtonEvent *event)
                      *newselect = (short)(newobject - (objinstptr *)topobject->plist);
 
 	          }
-	          if ((void *)mode == (void *)(1)) {
+	          if (op == XCF_Library_Copy) {
 
-		     /* key "c" pressed for "copy" */
+		     /* Key "c" pressed for "copy" (default binding) */
 
                      XDefineCursor(dpy, areastruct.areawin, COPYCURSOR);
-                     eventmode = COPY2_MODE;
+                     eventmode = COPY_MODE;
 #ifndef TCL_WRAPPER
                      xcAddEventHandler(areastruct.area, PointerMotionMask, False,
-			  (xcEventHandler)drag, NULL);
+			  (xcEventHandler)xlib_drag, NULL);
 #endif
 	          }
 	          else {
-                     eventmode = PRESS_MODE;
+                     eventmode = MOVE_MODE;
+	             areastruct.editcycle = 1;
 	          }
 #ifdef TCL_WRAPPER
+		  /* fprintf(stderr, "Creating event handler for xctk_drag: "); */
+		  /* printeventmode();		*/
                   Tk_CreateEventHandler(areastruct.area, PointerMotionMask,
 			(Tk_EventProc *)xctk_drag, NULL);
 #endif
+	 	  register_for_undo(XCF_Library_Pop, UNDO_MORE, areastruct.topinstance,
+				areastruct.selectlist, areastruct.selects);
                   catreturn();
                }
 
-               /* If button2 was pressed, select and stay in the catalog. */
-	       /* Could just do "objectselect" here, but would not cover   */
+               /* Select the closest element and stay in the catalog.	   */
+	       /* Could just do "select_element" here, but would not cover */
 	       /* the entire area in the directory surrounding the object. */
 
-	       else {
+	       else if (op == XCF_Select) {
 		  short newinst = (short)(libobj - (objinstptr *)topobject->plist);
 		  /* (ignore this object if it is already in the list of selects) */
 		  for (newselect = areastruct.selectlist; newselect <
@@ -1497,12 +1520,6 @@ void catbutton(u_int mode, XButtonEvent *event)
 	 }
       }
    }
-   /* If button3 was pressed, return without a selection. */
-
-   else {
-      eventmode = NORMAL_MODE;
-      catreturn();
-   }
 }
 
 /*------------------------------*/
@@ -1541,19 +1558,17 @@ void startcatalog(xcWidget w, pointertype libmod, caddr_t nulldata)
 		(areastruct.topinstance == xobjs.libtop[libmod])) return;
 
    if (libmod == FONTLIB) {
-      XDefineCursor (dpy, areastruct.areawin, CROSS);
-      if (eventmode == TEXT2_MODE)
+      XDefineCursor (dpy, areastruct.areawin, DEFAULTCURSOR);
+      if (eventmode == TEXT_MODE)
          eventmode = FONTCAT_MODE;
       else
-         eventmode = FONTCAT2_MODE;
+         eventmode = EFONTCAT_MODE;
    }
-#ifdef SCHEMA
    else if (eventmode == ASSOC_MODE) {
-      XDefineCursor (dpy, areastruct.areawin, CROSS);
+      XDefineCursor (dpy, areastruct.areawin, DEFAULTCURSOR);
    }
-#endif
    else if (libmod == PAGELIB || libmod == LIBLIB) {
-      XDefineCursor (dpy, areastruct.areawin, CROSS);
+      XDefineCursor (dpy, areastruct.areawin, DEFAULTCURSOR);
       eventmode = CATALOG_MODE;
    }
    else
@@ -1575,6 +1590,7 @@ void startcatalog(xcWidget w, pointertype libmod, caddr_t nulldata)
 
    areastruct.topinstance = xobjs.libtop[libmod];
    setpage(TRUE);
+   invalidate_graphics(topobject);
 
    /* draw the new screen */
 
