@@ -14,6 +14,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#ifndef XC_WIN32
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 
@@ -22,6 +23,7 @@
 #else
 #include "Xw/Xw.h"
 #endif
+#endif
 
 /*----------------------------------------------------------------------*/
 /* Local includes                                                       */
@@ -102,12 +104,10 @@ short execcommand(short pflags, char *cmdptr)
 	 areastruct.axeson = a ? True : False;
 	 toggle(GridAxesButton, &areastruct.axeson, NULL);
       }
-#ifdef SCHEMA
       else if (!strcmp(_STR, "xschema") || !strcmp(_STR, "schema")) {
 	 areastruct.schemon = a ? True : False;
 	 doxschema(OptionsEnableXSchemaButton, 1, NULL);
       }
-#endif
 #ifdef HAVE_XPM
       else if (!strcmp(_STR, "toolbar")) {
 	 areastruct.toolbar_on = a ? True : False;
@@ -173,7 +173,7 @@ short execcommand(short pflags, char *cmdptr)
 	 libnum = LIBRARY;
       else {
 	 if (libnum >= xobjs.numlibs)
-	    libnum = createlibrary();
+	    libnum = createlibrary(FALSE);
 	 else
 	    libnum += LIBRARY - 1;
       }
@@ -398,6 +398,7 @@ short readcommand(short mode, FILE *fd)
    }
 
    free(temp);
+   fclose(fd);
    return flags;
 }
 
@@ -496,14 +497,13 @@ void loadrcfile()
       addnewcolorentry(xc_alloccolor("Brown"));
    }
 
-#ifdef SCHEMA
    /* These colors must be enabled whether or not colors are overridden, */
    /* because they are needed by the schematic capture system.           */
 
    addnewcolorentry(xc_getlayoutcolor(LOCALPINCOLOR));
    addnewcolorentry(xc_getlayoutcolor(GLOBALPINCOLOR));
    addnewcolorentry(xc_getlayoutcolor(INFOLABELCOLOR));
-#endif
+   addnewcolorentry(xc_getlayoutcolor(RATSNESTCOLOR));
    addnewcolorentry(xc_getlayoutcolor(BBOXCOLOR));
 
 
