@@ -4,21 +4,35 @@
 /* These routines work only if ghostscript is on the system.		*/
 /*----------------------------------------------------------------------*/
 
+/* We use OpenGL functions to read and scale images if OpenGL is enabled */
+#ifndef OPENGL
+
 #undef GS_DEBUG
 /* #define GS_DEBUG */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#ifndef XC_WIN32
 #include <unistd.h>
+#endif
 #include <time.h>
+#ifndef XC_WIN32
 #include <sys/wait.h>   /* for waitpid() */
+#endif
 #include <sys/types.h>
 #include <signal.h>
 #include <fcntl.h>
+
+#ifndef XC_WIN32
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include <X11/Xatom.h>
+#else
+#ifdef TCL_WRAPPER
+#include <X11/Xatom.h>
+#endif
+#endif
 
 /*------------------------------------------------------------------------*/
 /* Local includes                                                         */
@@ -59,12 +73,22 @@ extern Pixmap dbuf;
 #define vfork fork
 #endif
 
+int gs_state;		/* Track state of the renderer */
+
+#define GS_INIT 0	/* Initial state; gs is idle. */
+#define GS_PENDING 1	/* Drawing in progress; gs is busy. */
+#define GS_READY 2	/* Drawing done; gs is waiting for "next". */
+
 /*------------------------------------------------------*/
 /* Global variable definitions				*/
 /*------------------------------------------------------*/
 
 Atom  gv, gvc, gvpage, gvnext, gvdone;
+#ifndef _MSC_VER
 pid_t gsproc = -1;	/* ghostscript process 			*/
+#else
+HANDLE gsproc = INVALID_HANDLE_VALUE;
+#endif
 int   fgs[2];		/* stdin pipe pair for ghostscript	*/
 Window mwin = 0;	/* "false" window hack to get gs	*/
 			/* process to capture ClientMessage	*/
@@ -77,14 +101,8 @@ Window mwin = 0;	/* "false" window hack to get gs	*/
 /* variable.							*/
 /*--------------------------------------------------------------*/
 
-void ghostinit()
+void ghostinit_local()
 {
-   gv = XInternAtom(dpy, "GHOSTVIEW", False);
-   gvc = XInternAtom(dpy, "GHOSTVIEW_COLORS", False);
-   gvpage = XInternAtom(dpy, "PAGE", False);
-   gvnext = XInternAtom(dpy, "NEXT", False);
-   gvdone = XInternAtom(dpy, "DONE", False);
-
    sprintf(_STR, "%ld %d %d %d %d %d %g %g %d %d %d %d",
 		0L, 0, 0, 0,
 		areastruct.width * 75 / 72,
@@ -96,6 +114,79 @@ void ghostinit()
    XChangeProperty(dpy, areastruct.areawin, gvc, XA_STRING, 8, PropModeReplace,
 	_STR, strlen(_STR));
    XSync(dpy, False);
+
+   gs_state = GS_INIT;
+}
+
+/*------------------------------------------------------*/
+/* Client message handler.  Called by either the Tk	*/
+/* client message handler (see below) or the Xt handler	*/
+/* (see xcircuit.c).					*/
+/*							*/
+/* Returns True if a client message was received from	*/
+/* the ghostscript process, False if not.		*/
+/*------------------------------------------------------*/
+
+Boolean render_client(XEvent *eventPtr)
+{
+   if (eventPtr->xclient.message_type == gvpage) {
+#ifdef GS_DEBUG
+      fprintf(stdout, "Xcircuit: Received PAGE message from ghostscript\n");
+#endif
+      mwin = eventPtr->xclient.data.l[0];
+      Wprintf("Background finished.");
+      XDefineCursor(dpy, areastruct.areawin, DEFAULTCURSOR);
+
+      /* Mark this as the most recently rendered background, so we don't	*/
+      /* have to render more than necessary.					*/
+      areastruct.lastbackground = xobjs.pagelist[areastruct.page]->background.name;
+      gs_state = GS_READY;
+      drawarea(areastruct.area, NULL, NULL);
+   }
+   else if (eventPtr->xclient.message_type == gvdone) {
+#ifdef GS_DEBUG
+      fprintf(stdout, "Xcircuit: Received DONE message from ghostscript\n");
+#endif
+      mwin = 0;
+      gs_state = GS_INIT;
+   } 
+   else {
+      return False;
+   }
+   return True;
+}
+
+/*------------------------------------------------------*/
+/* Tk client handler procedure for the above routine.	*/
+/*------------------------------------------------------*/
+
+#ifdef TCL_WRAPPER
+
+void handle_client(ClientData clientData, XEvent *eventPtr)
+{
+   if (render_client(eventPtr) == False)
+      fprintf(stderr, "Xcircuit: Received unknown client message\n");
+}
+
+#endif
+
+/*------------------------------------------------------*/
+/* Global initialization				*/
+/*------------------------------------------------------*/
+
+void ghostinit()
+{
+   gv = XInternAtom(dpy, "GHOSTVIEW", False);
+   gvc = XInternAtom(dpy, "GHOSTVIEW_COLORS", False);
+   gvpage = XInternAtom(dpy, "PAGE", False);
+   gvnext = XInternAtom(dpy, "NEXT", False);
+   gvdone = XInternAtom(dpy, "DONE", False);
+
+   ghostinit_local();
+
+#ifdef TCL_WRAPPER
+   Tk_CreateClientMessageHandler((Tk_ClientMessageProc *)handle_client);
+#endif
 }
 
 /*------------------------------------------------------*/
@@ -126,77 +217,21 @@ void send_client(Atom msg)
 
 void ask_for_next()
 {
+   if (gs_state != GS_READY) {
+      /* If we're in the middle of rendering something, halt	*/
+      /* immediately and start over.				*/
+      if (gs_state == GS_PENDING)
+	 reset_gs();
+      return;
+   }
    XSync(dpy, False);
+   gs_state = GS_PENDING;
    send_client(gvnext);
 #ifdef GS_DEBUG
    fprintf(stdout, "Xcircuit: Sent NEXT message to ghostscript\n");
 #endif
 }
 
-/*------------------------------------------------------*/
-/* Wait for the gs process to return "PAGE"		*/
-/* (Block until gs is finished)				*/
-/*------------------------------------------------------*/
-
-void wait_for_page()
-{
-   XEvent event;
-
-   for(;;) {
-      XNextEvent(dpy, &event);
-      if (event.type == ClientMessage) {
-	 if (event.xclient.message_type == gvpage) break;
-         else if (event.xclient.message_type == gvdone) {
-#ifdef GS_DEBUG
-	    fprintf(stdout, "Xcircuit: Received DONE message from ghostscript\n");
-#endif
-	    /* Catastrophic error in ghostscript */
-	    return;
-         }
-      } 
-      else if (event.type == KeyPress) {	/* Check for Ctrl-C break */
-	 XKeyEvent *keyevent = (XKeyEvent *)(&event);
-	 if (keyevent->state == 4 && keyevent->keycode == 54) {
-	    Wprintf("Ctrl-C break from render loop");
-	    break;
-	 }
-	 else {
-	    Fprintf(stderr, "state = %d, keycode = %d\n", keyevent->state,
-		keyevent->keycode);
-	    xcDispatchEvent(&event);
-	 }
-      }
-      else xcDispatchEvent(&event);	
-   }
-#ifdef GS_DEBUG
-   fprintf(stdout, "Xcircuit: Received PAGE message from ghostscript\n");
-#endif
-   mwin = event.xclient.data.l[0];
-}
-
-/*------------------------------------------------------*/
-
-void wait_for_done()
-{
-   XEvent event;
-
-   for(;;) {
-      XNextEvent(dpy, &event);
-      if (event.type == ClientMessage) {
-	 if (event.xclient.message_type == gvdone) break;
-#ifdef GS_DEBUG
-         else if (event.xclient.message_type == gvpage)
-	    fprintf(stdout, "Xcircuit: Received PAGE message from ghostscript\n");
-#endif
-      }
-      else xcDispatchEvent(&event);
-   }
-#ifdef GS_DEBUG
-   fprintf(stdout, "Xcircuit: Received DONE message from ghostscript\n");
-#endif
-   mwin = 0;
-}
-
 /*--------------------------------------------------------*/
 /* Start a ghostscript process and arrange the I/O pipes  */
 /* (Commented lines cause ghostscript to relay its stderr */
@@ -246,6 +281,58 @@ void start_gs()
    /* the background file to gs for rendering.				*/
    /* Here is not the place to do it.  Set up gs to take stdin as input.*/
 
+#ifdef _MSC_VER
+   if (gsproc == INVALID_HANDLE_VALUE) {
+	 STARTUPINFO st_info;
+	 PROCESS_INFORMATION pr_info;
+	 char cmd[4096];
+
+#ifdef HAVE_PUTENV
+   	 sprintf(env_str1, "DISPLAY=%s", XDisplayString(dpy));
+	 putenv(env_str1);
+#ifdef DOUBLEBUFFER
+   	 sprintf(env_str2, "GHOSTVIEW=%ld %ld", (long)areastruct.areawin, (long)bbuf);
+	 putenv(env_str2);
+#else
+	 sprintf(env_str2, "GHOSTVIEW=%ld", (long)areastruct.areawin);
+	 putenv(env_str2);
+#endif
+#else
+	 setenv("DISPLAY", XDisplayString(dpy), True);
+#ifdef DOUBLEBUFFER
+	 sprintf(_STR, "%ld %ld", (long)areastruct.areawin, (long)bbuf);
+	 setenv("GHOSTVIEW", _STR, True);
+#else
+ 	 sprintf(_STR, "%ld", (long)areastruct.areawin);
+	 setenv("GHOSTVIEW", _STR, True);
+#endif
+#endif
+
+	 SetHandleInformation((HANDLE)_get_osfhandle(fgs[1]), HANDLE_FLAG_INHERIT, 0);
+	 SetHandleInformation((HANDLE)_get_osfhandle(std_out[0]), HANDLE_FLAG_INHERIT, 0);
+#ifndef GS_DEBUG
+	 SetHandleInformation((HANDLE)_get_osfhandle(std_err[0]), HANDLE_FLAG_INHERIT, 0);
+#endif
+
+	 ZeroMemory(&st_info, sizeof(STARTUPINFO));
+	 ZeroMemory(&pr_info, sizeof(PROCESS_INFORMATION));
+	 st_info.cb = sizeof(STARTUPINFO);
+	 st_info.dwFlags = STARTF_USESTDHANDLES;
+	 st_info.hStdOutput = (HANDLE)_get_osfhandle(std_out[1]);
+	 st_info.hStdInput = (HANDLE)_get_osfhandle(fgs[0]);
+#ifndef GS_DEBUG
+	 st_info.hStdError = (HANDLE)_get_osfhandle(std_err[1]);
+#endif
+
+	 snprintf(cmd, 4095, "%s -dNOPAUSE -", GS_EXEC);
+	 if (CreateProcess(NULL, cmd, NULL, NULL, TRUE, 0, NULL, NULL, &st_info, &pr_info) == 0) {
+		 Wprintf("Error: ghostscript not running");
+		 return;
+	 }
+	 CloseHandle(pr_info.hThread);
+	 gsproc = pr_info.hProcess;
+   }
+#else
    if (gsproc < 0) {  /* Ghostscript is not running yet */
       gsproc = vfork();
       if (gsproc == 0) {		/* child process (gs) */
@@ -286,9 +373,9 @@ void start_gs()
 #endif
 #endif
 	 Flush(stderr);
-         execlp(GS_EXEC, "gs", "-dNOPAUSE", "-", NULL);
+         execlp(GS_EXEC, "gs", "-dNOPAUSE", "-", (char *)NULL);
 	 gsproc = -1;
-	 Fprintf(stderr, "Exec of gs failed\n");
+	 fprintf(stderr, "Exec of gs failed\n");
 	 return;
       }
       else if (gsproc < 0) {
@@ -296,6 +383,7 @@ void start_gs()
 	 return;			/* error condition */
       }
    }
+#endif
 }
 
 /*--------------------------------------------------------*/
@@ -360,7 +448,7 @@ void bg_get_bbox()
 
    fname = xobjs.pagelist[areastruct.page]->background.name;
    if ((fi = fopen(fname, "r")) == NULL) {
-      Fprintf(stderr, "Failure to open background file to get bounding box info\n");
+      fprintf(stderr, "Failure to open background file to get bounding box info\n");
       return;
    }
    parse_bg(fi, (FILE *)NULL);
@@ -410,11 +498,15 @@ void readbackground(FILE *fi)
    /* "@" denotes a temporary file */
    sprintf(file_in, "@%s/XXXXXX", xobjs.tempdir);
 
+#ifdef _MSC_VER
+   tfd = mktemp(file_in + 1);
+#else
    tfd = mkstemp(file_in + 1);
-   if (tfd == -1) Fprintf(stderr, "Error generating temporary filename\n");
+#endif
+   if (tfd == -1) fprintf(stderr, "Error generating temporary filename\n");
    else {
       if ((fbg = fdopen(tfd, "w")) == NULL) {
-	 Fprintf(stderr, "Error opening temporary file \"%s\"\n", file_in + 1);
+	 fprintf(stderr, "Error opening temporary file \"%s\"\n", file_in + 1);
       }
    }
 
@@ -425,7 +517,7 @@ void readbackground(FILE *fi)
 
    if (fbg != (FILE *)NULL) {
       fclose(fbg);
-      call_loadbg(file_in);
+      register_bg(file_in);
    }
    free(file_in);
 }
@@ -444,7 +536,7 @@ void savebackground(FILE *fo, char *psfilename)
    if (fname[0] == '@') fname++;
 
    if ((psf = fopen(fname, "r")) == NULL) {
-      Fprintf(stderr, "Error opening background file \"%s\" for reading.\n", fname);
+      fprintf(stderr, "Error opening background file \"%s\" for reading.\n", fname);
       return;
    }
 
@@ -457,12 +549,14 @@ void savebackground(FILE *fo, char *psfilename)
    fclose(psf);
 }
 
-/*------------------------------------------------------*/
-/* Load a generic (non-xcircuit) postscript file as the */
-/* background for the page.				*/
-/*------------------------------------------------------*/
+/*--------------------------------------------------------------*/
+/* Set up a page to render a PostScript image when redrawing.	*/
+/* This includes starting the ghostscript process if it has	*/
+/* not already been started.  This routine does not draw the	*/
+/* image, which is done on refresh.				*/
+/*--------------------------------------------------------------*/
 
-void call_loadbg(char *gsfile)
+void register_bg(char *gsfile)
 {
    if (gsproc < 0)
       start_gs();
@@ -474,14 +568,20 @@ void call_loadbg(char *gsfile)
    strcpy(xobjs.pagelist[areastruct.page]->background.name, gsfile);
 }
 
-/*--------------------------------------------------------*/
+/*------------------------------------------------------*/
+/* Load a generic (non-xcircuit) postscript file as the */
+/* background for the page.   This function is called	*/
+/* by the file import routine, and so it completes by	*/
+/* running zoomview(), which redraws the image and	*/
+/* the page.						*/
+/*------------------------------------------------------*/
 
 void loadbackground()
 {
-   call_loadbg(_STR2);
+   register_bg(_STR2);
    bg_get_bbox();
    updatepagebounds(topobject);
-   zoomview(NULL, NULL, NULL);
+   zoomview(areastruct.area, NULL, NULL);
 }
 
 /*------------------------------------------------------*/
@@ -497,7 +597,8 @@ void send_to_gs(char *text)
 }
 
 /*------------------------------------------------------*/
-/* Call ghostscript to render the background		*/
+/* Call ghostscript to render the background into the	*/
+/* pixmap buffer.					*/
 /*------------------------------------------------------*/
 
 int renderbackground()
@@ -522,25 +623,30 @@ int renderbackground()
 
    if (xobjs.pagelist[areastruct.page]->background.name == (char *)NULL)
       return -1;
-   else if (areastruct.lastbackground == xobjs.pagelist[areastruct.page]->background.name)
+   else if (areastruct.lastbackground
+		== xobjs.pagelist[areastruct.page]->background.name) {
       return 0;
+   }
 
    if (is_page(topobject) == -1)
       return -1;
 
-   ask_for_next();
-
    bgfile = xobjs.pagelist[areastruct.page]->background.name;
    if (*bgfile == '@') bgfile++;
 
-#ifdef GS_DEBUG
-   fprintf(stdout, "Rendering background from file \"%s\"\n", bgfile);
-#endif
+   /* Ask ghostscript to produce the next page */
+   ask_for_next();
+
+   /* Set the last background name to NULL; this will get the 	*/
+   /* value when the rendering is done.				*/
+
+   areastruct.lastbackground = NULL;
 
    /* write scale and position to ghostscript 		*/
    /* and tell ghostscript to run the requested file	*/
 
-/*   send_to_gs("/GSobj save def\n"); */
+   send_to_gs("/GSobj save def\n");
+   send_to_gs("/setpagedevice {pop} def\n");
    send_to_gs("gsave\n");
    sprintf(_STR, "%3.2f %3.2f translate\n", psxpos, psypos);
    send_to_gs(_STR);
@@ -548,31 +654,32 @@ int renderbackground()
    send_to_gs(_STR);
    sprintf(_STR, "(%s) run\n", bgfile);
    send_to_gs(_STR);
-/* send_to_gs("GSobj restore\n"); */
+   send_to_gs("GSobj restore\n");
    send_to_gs("grestore\n");
 
-   /* ask ghostscript to produce the next page */
-
-   XDefineCursor(dpy, areastruct.areawin, WAITFOR);
+#ifdef GS_DEBUG
+   fprintf(stdout, "Rendering background from file \"%s\"\n", bgfile);
+#endif
    Wprintf("Rendering background image.");
-   wait_for_page();
-   Wprintf("Background finished.");
-   XDefineCursor(dpy, areastruct.areawin, CROSS);
+   XDefineCursor(dpy, areastruct.areawin, WAITFOR);
 
-   /* Mark this as the most recently rendered background, so we don't	*/
-   /* have to render more than necessary.				*/
-   areastruct.lastbackground = xobjs.pagelist[areastruct.page]->background.name;
+   /* The page will be refreshed when we receive confirmation	*/
+   /* from ghostscript that the buffer has been rendered.	*/
 
    return 0;
 }
 
-/*----------------------------------------------------------*/
-/* Copy the rendered background pixmap to the buffer pixmap */
-/*----------------------------------------------------------*/
+/*------------------------------------------------------*/
+/* Copy the rendered background pixmap to the window.	*/
+/*------------------------------------------------------*/
 
 int copybackground()
 {
-   /* only draw on a top-level page */
+   /* Don't copy if the buffer is not ready to use */
+   if (gs_state != GS_READY)
+      return -1;
+
+   /* Only draw on a top-level page */
    if (is_page(topobject) == -1)
       return -1;
 
@@ -588,25 +695,37 @@ int copybackground()
 }
 
 /*------------------------------------------------------*/
-/* Exit ghostscript. . . gently				*/
+/* Exit ghostscript. . . not so gently			*/
 /*------------------------------------------------------*/
 
 int exit_gs()
 {
+#ifdef _MSC_VER
+   if (gsproc == INVALID_HANDLE_VALUE) return -1;	/* gs not running */
+#else
    if (gsproc < 0) return -1;	/* gs not running */
+#endif
 
-   send_to_gs("quit\n");
-   ask_for_next();
 #ifdef GS_DEBUG
    fprintf(stdout, "Waiting for gs to exit\n");
 #endif
+#ifndef _MSC_VER
+   kill(gsproc, SIGKILL);
    waitpid(gsproc, NULL, 0);      
+#else
+   TerminateProcess(gsproc, -1);
+#endif
 #ifdef GS_DEBUG
    fprintf(stdout, "gs has exited\n");
 #endif
 
    mwin = 0;
+#ifdef _MSC_VER
+   gsproc = INVALID_HANDLE_VALUE;
+#else
    gsproc = -1;
+#endif
+   gs_state = GS_INIT;
 
    return 0;
 }
@@ -617,13 +736,19 @@ int exit_gs()
 
 int reset_gs()
 {
+#ifdef _MSC_VER
+   if (gsproc == INVALID_HANDLE_VALUE) return -1;
+#else
    if (gsproc < 0) return -1;
+#endif
 
    exit_gs();
-   ghostinit();
+   ghostinit_local();
    start_gs();
 
    return 0;
 }
 
+#endif /* !OPENGL */
+
 /*----------------------------------------------------------------------*/
