@@ -10,21 +10,20 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#if defined(DARWIN)
-#include <sys/malloc.h>
-#else
-#include <malloc.h>
-#endif
 
+#ifndef XC_WIN32
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
+#endif
 
 #ifdef TCL_WRAPPER 
 #include <tk.h>
 #else
+#ifndef XC_WIN32
 #include "Xw/Xw.h"
 #include "Xw/MenuBtn.h"
 #endif
+#endif
 
 /*-------------------------------------------------------------------------*/
 /* Local includes							   */
@@ -57,7 +56,6 @@ extern char	  _STR[150];
 extern char	  _STR2[250];
 
 /*-------------------------------------------------------------------------*/
-#ifdef SCHEMA
 
 extern xcWidget wsymb, wschema;
 
@@ -65,23 +63,24 @@ extern xcWidget wsymb, wschema;
 /* Menu calls (procedure wrappers)			  */
 /*--------------------------------------------------------*/
 
-void callgennet(xcWidget w, pointertype mode, caddr_t calldata)
+void callwritenet(xcWidget w, pointertype mode, caddr_t calldata)
 {
    switch(mode) {
      case 0:
-        gennet("spice", "spc");
+        writenet(topobject, "spice", "spc");
 	break;
      case 1:
-        gennet("sim", "sim");
+        writenet(topobject, "sim", "sim");
 	break;
      case 2:
-        gennet("pcb", "pcbnet");
+        writenet(topobject, "pcb", "pcbnet");
 	break;
      case 3:
-        gennet("flatspice", "fspc");
+        writenet(topobject, "flatspice", "fspc");
 	break;
      case 4:
-	gennet("idxpcb", "");	/* Auto-numbering:  no output generated */
+	/* Auto-numbering:  no output generated */
+	writenet(topobject, "idxpcb", "");
 	break;
    }
 }
@@ -94,7 +93,9 @@ void callgennet(xcWidget w, pointertype mode, caddr_t calldata)
 
 void doxschema(xcWidget w, pointertype mode, caddr_t calldata)
 {
+#ifndef TCL_WRAPPER
    Arg wargs[1];
+#endif
 
    if (areastruct.schemon) {
       areastruct.schemon = False;
@@ -125,6 +126,49 @@ void doxschema(xcWidget w, pointertype mode, caddr_t calldata)
    if (mode) drawarea(NULL, NULL, NULL);
 }
 
+/*----------------------------------------------------------------------*/
+/* Find the page object and instance with the indicated name.		*/
+/*----------------------------------------------------------------------*/
+
+objectptr NameToPageObject(char *objname, objinstptr *ret_inst, int *ret_page)
+{
+   int i;
+
+   for (i = 0; i < xobjs.pages; i++) {
+      if (xobjs.pagelist[i]->pageinst == NULL) continue;
+      if (!strcmp(objname, xobjs.pagelist[i]->pageinst->thisobject->name)) {
+	 if (ret_inst) *ret_inst = xobjs.pagelist[i]->pageinst;
+	 if (ret_page) *ret_page = i;
+	 return xobjs.pagelist[i]->pageinst->thisobject;
+      }
+   }
+   return NULL;
+}
+
+/*----------------------------------------------------------------------*/
+/* Find the object with the indicated name.				*/
+/*----------------------------------------------------------------------*/
+
+objectptr NameToObject(char *objname, objinstptr *ret_inst, Boolean dopages)
+{
+   int i;
+   liblistptr spec;
+
+   for (i = 0; i < xobjs.numlibs; i++) {
+      for (spec = xobjs.userlibs[i].instlist; spec != NULL; spec = spec->next) {
+         if (!strcmp(objname, spec->thisinst->thisobject->name)) {
+	    if (ret_inst) *ret_inst = spec->thisinst;
+	    return spec->thisinst->thisobject;
+	 }
+      }
+   }
+
+   if (dopages)
+      return NameToPageObject(objname, ret_inst, NULL);
+   else
+      return NULL;
+}
+
 /*------------------------------------------------------------------------*/
 /* Ensure that a page name is unique (required for schematic association) */
 /*------------------------------------------------------------------------*/
@@ -193,30 +237,28 @@ void startconnect(xcWidget button, caddr_t clientdata, caddr_t calldata)
 {
    if (areastruct.selects > 0)
       connectivity(button, clientdata, calldata);
-   else if (eventmode == NORMAL_MODE) {
-      Wprintf("Click button1 on wire to see connectivity.");
-      eventmode = CONNECT_MODE;
-   }
 }
 
 /*----------------------------------------------------------------------*/
 /* connectivity():  Find electrical connections into a node.		*/
 /* (does recursive search on object heirarchy)				*/
-/* Returns net number found (0 if no network found, -1 if error)	*/
 /*----------------------------------------------------------------------*/
 
-int connectivity(xcWidget button, caddr_t clientdata, caddr_t calldata)
+void connectivity(xcWidget button, caddr_t clientdata, caddr_t calldata)
 {
-   short *gsel;
-   genericptr ggen;
-   int depth, netid = -1;
+   short *gsel = NULL;
+   selection *rselect = NULL, *nextselect;
+   genericptr ggen = NULL;
+   Genericlist *netlist = NULL;
+   int depth, lbus, netid, subnetid;
+   buslist *sbus;
    pushlistptr seltop, nextptr;
-   objectptr nettop, thisobj;
+   objectptr nettop, thisobj, pschem;
    char *snew;
    stringpart *ppin;
 
    /* erase any existing highlighted network */
-   highlightnet(topobject, areastruct.topinstance, -1, 0);
+   highlightnetlist(topobject, areastruct.topinstance, 0);
 
    seltop = (pushlistptr)malloc(sizeof(pushlist));
    seltop->thisinst = areastruct.topinstance;
@@ -237,52 +279,81 @@ int connectivity(xcWidget button, caddr_t clientdata, caddr_t calldata)
 	    if (!nonnetwork(gpoly)) break;
 	 }
       }
-      if (gsel == areastruct.selectlist + areastruct.selects) {
-	 Wprintf("Selected element is not part of a valid network.");
+   }
+   if ((areastruct.selects == 0) || (gsel == areastruct.selectlist
+		+ areastruct.selects)) {
+      rselect = recurselect(POLYGON | LABEL | OBJINST, MODE_CONNECT, &seltop);
+      /* Should look at the top scorer, when scoring has been implemented */
+      if (rselect && (rselect->selects > 0)) {
+	 for (nextselect = rselect; (nextselect->next != NULL) &&
+		(nextselect->selects > 0); nextselect = nextselect->next);
+         ggen = *(nextselect->thisinst->thisobject->plist + *(nextselect->selectlist));
+	 while (rselect != NULL) {
+	    nextselect = rselect->next;
+            free(rselect->selectlist);
+	    free(rselect);
+	    rselect = nextselect;
+	 }
       }
    }
-   else
-      ggen = recurselect(POLYGON | LABEL, &seltop);
 
-   /* Determine the netid and the topmost object in which that	*/
+   /* Determine the net and the topmost object in which that	*/
    /* net appears.  Then build the transformation matrix down	*/
    /* to that object.  highlightnet() should end by popping the */
    /* entire matrix stack.					*/
 
    if (ggen != NULL) {
       if (checkvalid(topobject) == -1) {
-         destroynets();
-         createnets();
+         destroynets(topobject);
+         createnets(areastruct.topinstance);
       }
-      if ((netid = is_resolved(&ggen, seltop, &nettop)) != 0) {
-         /* Fprintf(stdout, "Net ID is %d in object %s\n", netid, nettop->name); */
+      if ((netlist = is_resolved(&ggen, seltop, &nettop)) != NULL) {
          depth = pushnetwork(seltop, nettop);
          /* Fprintf(stdout, ">> Pushed network %d levels deep\n", depth); */
          nextptr = seltop;
          while (nextptr->thisinst->thisobject != nettop)
 	    nextptr = nextptr->next;
 
-	 nextptr->thisinst->thisobject->highlight.netid = netid;
+	 nextptr->thisinst->thisobject->highlight.netlist = netlist;
 	 nextptr->thisinst->thisobject->highlight.thisinst = nextptr->thisinst;
+	 highlightnetlist(nettop, nextptr->thisinst, 1);
 	    
-         highlightnet(nettop, nextptr->thisinst, netid, 1);
-
          /* pop the matrix stack */
          while (depth-- > 0) 
 	    UPopCTM();
 
+	 /* get the primary schematic */
+	 pschem = (nettop->schemtype == SECONDARY) ? nettop->symschem : nettop;
+
 	 /* print the net name to the message window */
 
-	 ppin = nettopin(netid, nettop, NULL);
-         snew = textprint(ppin, areastruct.topinstance);
-
-	 sprintf(_STR, "Network is \"%s\" in %s", snew, nettop->name);
-	 Wprintf(_STR);
+	 if (netlist->subnets == 0) {
+	    ppin = nettopin(netlist->net.id, pschem, NULL);
+            snew = textprint(ppin, areastruct.topinstance);
+	    sprintf(_STR2, "Network is \"%s\" in %s", snew, nettop->name);
+            free(snew);
+	 }
+	 else {
+	    char *sptr;
+	    sprintf(_STR2, "Network(s): ");
+	    sptr = _STR2 + strlen(_STR2);
+	    for (lbus = 0; lbus < netlist->subnets; lbus++) {
+	       sbus = netlist->net.list + lbus;
+	       netid = sbus->netid;
+	       subnetid = sbus->subnetid;
+	       ppin = nettopin(netid, pschem, NULL);
+               snew = textprintsubnet(ppin, areastruct.topinstance, subnetid);
+	       sprintf(sptr, "%s ", snew);
+	       sptr += strlen(snew) + 1;
+	       free(snew);
+	    }
+	    sprintf(sptr, "in %s", nettop->name);
+	 }
+	 Wprintf(_STR2);
 
 #ifdef TCL_WRAPPER
 	 Tcl_SetObjResult(xcinterp, Tcl_NewStringObj(snew, strlen(snew)));
 #endif
-         free(snew);
       }
       else
 	 Wprintf("Selected element is not part of a valid network.");
@@ -299,7 +370,6 @@ int connectivity(xcWidget button, caddr_t clientdata, caddr_t calldata)
       free(seltop);
       seltop = nextptr;
    }
-   return netid;
 }
 
 /*--------------------------------------------------------------*/
@@ -317,12 +387,12 @@ Boolean setobjecttype(objectptr cschem)
 
    /* Apply only to schematic objects */
 
-   if (cschem->schemtype != SCHEMATIC) {
+   if ((cschem->schemtype != PRIMARY) && (cschem->schemtype != SECONDARY)) {
       if (cschem->schemtype == FUNDAMENTAL)
 	 cschem->schemtype = SYMBOL;
       if (cschem->symschem == NULL) {
          for (cgen = cschem->plist; cgen < cschem->plist + cschem->parts; cgen++) {
-            if ((*cgen)->type == LABEL) {
+            if (IS_LABEL(*cgen)) {
                clab = TOLABEL(cgen);
 	       if (clab->pin == INFO) {
 	          cschem->schemtype = FUNDAMENTAL;
@@ -405,7 +475,7 @@ void dopintype(xcWidget w, pointertype mode, caddr_t calldata)
       }
 
    if (savetype >= 0) {
-      objectdeselect();
+      unselect_all();
       drawarea(NULL, NULL, NULL);
       Wprintf(typestr);
    }
@@ -441,7 +511,7 @@ void setsymschem()
    /* Set menu items appropriately for this object */
 
    if (topobject->symschem != NULL) {
-      if (topobject->schemtype == SCHEMATIC) {
+      if (topobject->schemtype == PRIMARY || topobject->schemtype == SECONDARY) {
          XtSetArg(aargs[0], XtNlabel, "Go To Symbol");
          XtSetArg(bargs[0], XtNlabel, "Disassociate Symbol");
       }
@@ -451,7 +521,7 @@ void setsymschem()
       }
    }
    else {
-      if (topobject->schemtype == SCHEMATIC) {
+      if (topobject->schemtype == PRIMARY || topobject->schemtype == SECONDARY) {
          XtSetArg(aargs[0], XtNlabel, "Make Matching Symbol");
          XtSetArg(bargs[0], XtNlabel, "Associate with Symbol");
       }
@@ -465,7 +535,7 @@ void setsymschem()
 
    /* Set colors on the symbol and schematic buttons */
 
-   if (topobject->schemtype == SCHEMATIC) {
+   if (topobject->schemtype == PRIMARY || topobject->schemtype == SECONDARY) {
       if (topobject->symschem == NULL) {
          XtSetArg(aargs[0], XtNbackground, OFFBUTTONCOLOR);
 	 XtSetArg(aargs[1], XtNforeground, OFFBUTTONCOLOR);
@@ -523,55 +593,47 @@ int findpageobj(objectptr pobj)
    return -1;
 }
 
-/*-------------------------------------------------------*/
-/* Recursively find all sub-circuits associated with the */
-/* top-level circuit and set their filenames to be the	 */
-/* same.						 */
-/*							 */
-/* Avoid possible recursion problems by limiting the	 */
-/* number of recursion levels.  Presumably no circuit	 */
-/* would have more than several hundred hierarchical 	 */
-/* levels.						 */
-/*-------------------------------------------------------*/
+/*------------------------------------------------------*/
+/* Enumerate all of the pages which are subschematics   */
+/* of "toppage" and return the result in the list	*/
+/* passed as a pointer.  The list is assumed to be	*/
+/* already allocated, and equal to the total number of	*/
+/* pages (xobjs.pages).					*/
+/*							*/
+/* Avoid possible recursion problems by limiting the	*/
+/* number of recursion levels.  Presumably no circuit	*/
+/* would have more than several hundred hierarchical 	*/
+/* levels.						*/
+/*------------------------------------------------------*/
 
-int findsubschems(int toppage, objectptr cschem, int level)
+int findsubschems(int toppage, objectptr cschem, int level, short *pagelist)
 {
    genericptr *cgen;
 
-   if (level == HIERARCHY_LIMIT) return -1;
+   if (level == HIERARCHY_LIMIT) return -1;	/* sanity check */
 
    for (cgen = cschem->plist; cgen < cschem->plist + cschem->parts; cgen++) {
-      if ((*cgen)->type == OBJECT) {
+      if (IS_OBJINST(*cgen)) {
 	 objectptr cobj = TOOBJINST(cgen)->thisobject;
 
 	 if (cobj->symschem != NULL) {
 	    int pageno = findpageobj(cobj->symschem);
-	    /* Fprintf(stdout, "Symbol %s has schematic at page %d\n",
-			cobj->name, pageno); */
-	    if (pageno < 0) {	/* This shouldn't happen (hopefully) */
-	       Wprintf("Error: disconnected schematic!\n");
-	    }
-	    else {
-	       if (xobjs.pagelist[pageno]->filename != NULL)
-		  free(xobjs.pagelist[pageno]->filename);
-	       xobjs.pagelist[pageno]->filename =
-			strdup(xobjs.pagelist[toppage]->filename);
-	    }
+	    if ((pageno >= 0) && (pageno < xobjs.pages))
+		pagelist[pageno]++;
 
 	    /* A symbol on its own schematic page is allowed for clarity */
 	    /* of the schematic, but this cannot be a functional part of */
 	    /* the schematic circuit!					 */
 	    
 	    if (cobj->symschem != cschem) {
-	       /* Fprintf(stdout, "Searching for dependencies of symbol %s\n",
-			cobj->name); */
-	       if (findsubschems(toppage, cobj->symschem, level + 1) == -1)
+	       if (findsubschems(toppage, cobj->symschem,
+				level + 1, pagelist) == -1)
 		  return -1;
 	    }
 	 }
 	 else if (cobj->schemtype != FUNDAMENTAL && cobj->schemtype != TRIVIAL) {
 	    /* Check symbols acting as their own schematics */
-	    if (findsubschems(toppage, cobj, level + 1) == -1)
+	    if (findsubschems(toppage, cobj, level + 1, pagelist) == -1)
 	       return -1;
 	 }
       }
@@ -579,48 +641,47 @@ int findsubschems(int toppage, objectptr cschem, int level)
    return 0;
 }
 
-/*--------------------------------------------------------*/
-/* Copy label to corresponding schematic/symbol		  */
-/*--------------------------------------------------------*/
+/*-------------------------------------------------------*/
+/* Recursively find all sub-circuits associated with the */
+/* top-level circuit and set their filenames to be the	 */
+/* same.						 */
+/*-------------------------------------------------------*/
 
-void copypinlabel(labelptr pinlab)
+void collectsubschems(int toppage)
 {
-   labelptr *newlabel, tlabel;
-   genericptr *tgen;
-   objectptr schemobj = topobject->symschem;
+   int loctop, level = 0;
+   objectptr cschem;
+   short *pagelist, pageno;
+   Pagedata *curpage;
 
-   if (areastruct.schemon && schemobj != NULL && (pinlab->pin == LOCAL)) {
-
-      /* If this pin already exists, don't copy it */
-
-      for (tgen = schemobj->plist; tgen < schemobj->plist + schemobj->parts;
-		tgen++) {
-         if ((*tgen)->type == LABEL) {
-	    tlabel = TOLABEL(tgen);
-	    if (!stringcomp(tlabel->string, pinlab->string)) return;
-         }
-      }
-
-      NEW_LABEL(newlabel, schemobj);
-      (*newlabel)->pin = pinlab->pin;
-      (*newlabel)->rotation = pinlab->rotation;
-      (*newlabel)->justify = pinlab->justify;
-      (*newlabel)->color = pinlab->color;
-      (*newlabel)->scale = pinlab->scale;
-      (*newlabel)->string = stringcopy(pinlab->string);
-      (*newlabel)->num_params = 0;
-      (*newlabel)->passed = NULL;
-
-      /* place label just outside bounding box, then recompute bbox */
-
-      (*newlabel)->position.x = schemobj->bbox.lowerleft.x + (schemobj->bbox.width >> 1);
-      (*newlabel)->position.y = schemobj->bbox.lowerleft.y - TEXTHEIGHT
-	   * (*newlabel)->scale;
-      schemobj->parts++;
-      incr_changes(schemobj);
-      schemobj->valid = False;
-      singlebbox(ENDPART);
+   loctop = toppage;
+   curpage = xobjs.pagelist[loctop];
+   if (curpage->pageinst == NULL) return;
+   cschem = curpage->pageinst->thisobject;
+   if (cschem->schemtype == SECONDARY) {
+      cschem = cschem->symschem;
+      loctop = is_page(cschem);
+      if (loctop < 0) return;
+      curpage = xobjs.pagelist[loctop];
    }
+
+   pagelist = (short *)malloc(xobjs.pages * sizeof(short));
+
+   for (pageno = 0; pageno < xobjs.pages; pageno++)
+      pagelist[pageno] = 0;
+
+   findsubschems(loctop, cschem, 0, pagelist);
+
+   for (pageno = 0; pageno < xobjs.pages; pageno++) {
+      if (pageno == loctop) continue;
+      if (pagelist[pageno] > 0) {
+	  if (xobjs.pagelist[pageno]->filename != NULL)
+	     free(xobjs.pagelist[pageno]->filename);
+	  xobjs.pagelist[pageno]->filename =
+	     strdup(xobjs.pagelist[loctop]->filename);
+      }
+   } 
+   free((char *)pagelist);
 }
 
 /*-------------------------------------------------------*/
@@ -638,7 +699,7 @@ int checkschem(objectptr thisobj, char *cname)
    objectptr *tlib;
    short i, j;
 
-   if (areastruct.schemon == False || thisobj->symschem != NULL) return 0;
+   if ((areastruct.schemon == False) || (thisobj->symschem != NULL)) return 0;
 
    for (i = 0; i < xobjs.numlibs; i++) {
       for (j = 0; j < xobjs.userlibs[i].number; j++) {
@@ -646,7 +707,7 @@ int checkschem(objectptr thisobj, char *cname)
 
          if (!strcmp(cname, (*tlib)->name)) {
 	    thisobj->symschem = (*tlib);
-	    thisobj->schemtype = SCHEMATIC;
+	    thisobj->schemtype = PRIMARY;
 	    (*tlib)->symschem = thisobj;
 	    (*tlib)->schemtype = SYMBOL;
 	    return 1;
@@ -675,7 +736,7 @@ int checksym(objectptr symobj, char *cname)
 	    symobj->symschem = checkpage;
 	    symobj->schemtype = SYMBOL;
 	    checkpage->symschem = symobj;
-	    checkpage->schemtype = SCHEMATIC;
+	    checkpage->schemtype = PRIMARY;
 	    return 1;
 	 }
       }
@@ -685,8 +746,7 @@ int checksym(objectptr symobj, char *cname)
 
 /*----------------------------------------------------------------------*/
 /* Find location of corresponding pin in symbol/schematic and change it */
-/* to the text of the indicated label.  If "newlabel" is NULL, then	*/
-/* change the other label from pin to normal text.			*/
+/* to the text of the indicated label.					*/
 /*----------------------------------------------------------------------*/
 
 void changeotherpins(labelptr newlabel, stringpart *oldstring)
@@ -698,43 +758,55 @@ void changeotherpins(labelptr newlabel, stringpart *oldstring)
    if (other == NULL) return;
 
    for (tgen = other->plist; tgen < other->plist + other->parts; tgen++) {
-      if ((*tgen)->type == LABEL) {
+      if (IS_LABEL(*tgen)) {
 	 tlab = TOLABEL(tgen);
+	 if (tlab->pin != LOCAL) continue;
 	 if (!stringcomp(tlab->string, oldstring)) {
 	    if (newlabel != NULL) {
 	       free(tlab->string);
 	       tlab->string = stringcopy(newlabel->string);
 	    }
-	    else
-	       tlab->pin = NORMAL;
 	 }
       }
    }
 }
 
 /*----------------------------------------------------------------------*/
-/* Swap object schematic and symbol pages.   				*/
-/*  mode = 0 disallows creation of a new schematic or symbol; i.e., if	*/
-/*  there is no corresponding schematic/symbol, nothing happens.	*/
+/* Xt wrapper for swapschem()						*/
 /*----------------------------------------------------------------------*/
 
-void swapschem(xcWidget w, pointertype mode, caddr_t calldata)
+void xlib_swapschem(xcWidget w, pointertype mode, caddr_t calldata)
+{
+   swapschem((int)mode, -1);
+}
+
+/*----------------------------------------------------------------------*/
+/* Swap object schematic and symbol pages.   				*/
+/*  allow_create = 0 disallows creation of a new schematic or symbol;	*/
+/*  i.e., if there is no corresponding schematic/symbol, nothing	*/
+/*  happens.								*/
+/*----------------------------------------------------------------------*/
+
+void swapschem(int allow_create, int libnum)
 {
    objectptr savepage = topobject;
    labelptr  *pinlab;
    genericptr *plab;
    Boolean lflag;
    pushlistptr stacktop;
+   short loclibnum = libnum;
 
    if (areastruct.schemon == False) return;
+   if (libnum == -1) loclibnum = USERLIB - LIBRARY;
 
-   if (eventmode == PRESS_MODE || eventmode == COPY2_MODE)
-      objectdelete(NORMAL);
+   if (eventmode == MOVE_MODE || eventmode == COPY_MODE)
+      standard_element_delete(NORMAL);
 
-   /* Create symbol or schematic, if allowed by mode */
+   /* Create symbol or schematic, if allowed by allow_create */
 
-   if ((topobject->symschem == NULL) && (mode != 0)) {
-      if (topobject->schemtype != SCHEMATIC) {
+   if ((topobject->symschem == NULL) && (allow_create != 0)
+		&& (topobject->schemtype != SECONDARY)) {
+      if (topobject->schemtype != PRIMARY) {
 	 int tpage;
 
 	 /* create a new page for the new schematic */
@@ -745,36 +817,34 @@ void swapschem(xcWidget w, pointertype mode, caddr_t calldata)
 	 /* Push the current instance onto the push stack */
 	 /* Change the page without destroying the pushlist */
 
-	 push_stack(&xobjs.stack, areastruct.topinstance);
-	 stacktop = xobjs.stack;
-	 xobjs.stack = NULL;
+	 push_stack(&areastruct.stack, areastruct.topinstance);
+	 stacktop = areastruct.stack;
+	 areastruct.stack = NULL;
          changepage(tpage);
-	 xobjs.stack = stacktop;
+	 areastruct.stack = stacktop;
       }
       else {
 	 objectptr *newobject;
-	 short libnum = USERLIB - LIBRARY;
 
 	 /* create a new library object for the new symbol */
 
-	 xobjs.userlibs[libnum].library = (objectptr *)
-		realloc(xobjs.userlibs[libnum].library,
-		++xobjs.userlibs[libnum].number * sizeof(objectptr));
-	 newobject = xobjs.userlibs[libnum].library
-		+ xobjs.userlibs[libnum].number - 1;
+	 xobjs.userlibs[loclibnum].library = (objectptr *)
+		realloc(xobjs.userlibs[loclibnum].library,
+		++xobjs.userlibs[loclibnum].number * sizeof(objectptr));
+	 newobject = xobjs.userlibs[loclibnum].library
+		+ xobjs.userlibs[loclibnum].number - 1;
          *newobject = (objectptr) malloc(sizeof(object));
 	 initmem(*newobject);
 	 (*newobject)->schemtype = SYMBOL;
 	 (*newobject)->hidden = False;
-	 (*newobject)->devname = NULL;
 
 	 incr_changes(*newobject);
 
 	 /* Generate a library instance for this object and set the */
 	 /* top instance to point to it.			    */
 
-	 push_stack(&xobjs.stack, areastruct.topinstance);
-	 areastruct.topinstance = addtoinstlist(libnum, *newobject, FALSE);
+	 push_stack(&areastruct.stack, areastruct.topinstance);
+	 areastruct.topinstance = addtoinstlist(loclibnum, *newobject, FALSE);
 
 	 /* Generate the default bounding box for a size-zero object */
 	 calcbbox(areastruct.topinstance);
@@ -793,7 +863,7 @@ void swapschem(xcWidget w, pointertype mode, caddr_t calldata)
 
       for (plab = savepage->plist; plab < savepage->plist + savepage->parts;
 		plab++) {
-	 if ((*plab)->type == LABEL) {
+	 if (IS_LABEL(*plab)) {
 	    genericptr *tgen;
 	    labelptr tlab, lpin = (labelptr)*plab;
 
@@ -804,7 +874,7 @@ void swapschem(xcWidget w, pointertype mode, caddr_t calldata)
 	       lflag = False;
                for (tgen = topobject->plist; tgen <
 			topobject->plist + topobject->parts; tgen++) {
-		  if ((*tgen)->type == LABEL) {
+		  if (IS_LABEL(*tgen)) {
 		     tlab = TOLABEL(tgen);
 	             if (!stringcomp(tlab->string, lpin->string)) lflag = True;
 		  }
@@ -819,7 +889,6 @@ void swapschem(xcWidget w, pointertype mode, caddr_t calldata)
 	       (*pinlab)->justify = areastruct.justify; 
 	       (*pinlab)->position.x = 0;
 	       (*pinlab)->position.y = topobject->parts * (TEXTHEIGHT + 10);
-	       (*pinlab)->num_params = 0;
 	       (*pinlab)->passed = NULL;
 	       u2u_snap(&((*pinlab)->position));
 	       (*pinlab)->string = stringcopy(lpin->string);
@@ -831,16 +900,17 @@ void swapschem(xcWidget w, pointertype mode, caddr_t calldata)
       calcbbox(areastruct.topinstance);
 
       /* Recreate the user library with the new symbol */
-      if (savepage->schemtype != SYMBOL) composelib(USERLIB);
+      if (savepage->schemtype != SYMBOL) composelib(loclibnum + LIBRARY);
    }
    else if (topobject->symschem != NULL) {
 
       /* If symschem matches the last entry on the push stack, then we	*/
       /* pop; otherwise, we push.					*/
 
-      if (xobjs.stack && xobjs.stack->thisinst->thisobject == topobject->symschem) {
-	 areastruct.topinstance = xobjs.stack->thisinst;
-	 pop_stack(&xobjs.stack);
+      if (areastruct.stack && areastruct.stack->thisinst->thisobject
+			== topobject->symschem) {
+	 areastruct.topinstance = areastruct.stack->thisinst;
+	 pop_stack(&areastruct.stack);
       }
       else {
 	 int p;
@@ -876,7 +946,7 @@ void swapschem(xcWidget w, pointertype mode, caddr_t calldata)
 	    }
          }
 	       
-	 push_stack(&xobjs.stack, areastruct.topinstance);
+	 push_stack(&areastruct.stack, areastruct.topinstance);
 	 areastruct.topinstance = syminst;
       }
    }
@@ -891,6 +961,8 @@ void swapschem(xcWidget w, pointertype mode, caddr_t calldata)
    setsymschem();
 }
 
+#ifndef TCL_WRAPPER
+
 /*----------------------------------------------------------------------*/
 /* Wrapper for swapschem() when generating a new symbol.		*/
 /*----------------------------------------------------------------------*/
@@ -901,7 +973,7 @@ void makesymbol(xcWidget w, caddr_t calldata)
 
    strcpy(topobject->name, _STR2);
    checkname(topobject);
-   swapschem(w, (pointertype)1, calldata);
+   swapschem(1, -1);
 }
 
 /*----------------------------------------------------------------------*/
@@ -919,7 +991,7 @@ void dobeforeswap(xcWidget w, caddr_t clientdata, caddr_t calldata)
 
    /* Check for requirement to change the name before creating the symbol */
 
-   if ((topobject->symschem == NULL) && (topobject->schemtype == SCHEMATIC)
+   if ((topobject->symschem == NULL) && (topobject->schemtype == PRIMARY)
 		&& (strstr(topobject->name, "Page ") != NULL)) {
 
       /* Get a name for the new object */
@@ -930,9 +1002,11 @@ void dobeforeswap(xcWidget w, caddr_t clientdata, caddr_t calldata)
       popupprompt(w, "Enter name for new object:", "\0", makesymbol, popdata, NULL);
    }
    else
-      swapschem(w, (pointertype)1, calldata);
+      swapschem(1, -1);
 }
 
+#endif /* !TCL_WRAPPER */
+
 /*------------------------------------------*/
 /* Disassociate a symbol from its schematic */
 /*------------------------------------------*/
@@ -965,17 +1039,20 @@ void startschemassoc(xcWidget w, pointertype mode, caddr_t calldata)
    else if ((topobject->symschem != NULL) && (mode == 0)) {
       Wprintf("Refusing to undo current association.");
    }
+   else if (topobject->schemtype == SECONDARY) {
+      Wprintf("Cannot attach symbol to a secondary schematic page.");
+   }
    else {
       eventmode = ASSOC_MODE;
-      if (topobject->schemtype == SCHEMATIC) {
+      if (topobject->schemtype == PRIMARY) {
 	 /* Find a symbol to associate */
 	 startcatalog(w, LIBLIB, NULL);
-	 Wprintf("Click on library page, then symbol to associate.");
+	 Wprintf("Select library page, then symbol to associate.");
       }
       else {
 	 /* Find a schematic (page) to associate */
 	 startcatalog(w, PAGELIB, NULL);
-	 Wprintf("Click on schematic page to associate.");
+	 Wprintf("Select schematic page to associate.");
       }
    }
 }
@@ -984,13 +1061,14 @@ void startschemassoc(xcWidget w, pointertype mode, caddr_t calldata)
 /* Callback procedures on the schematic/symbol buttons.		*/
 /*--------------------------------------------------------------*/
 
-void schemassoc(objectptr schemobj, objectptr symbolobj)
+Boolean schemassoc(objectptr schemobj, objectptr symbolobj)
 {
-   if (eventmode != ASSOC_MODE) {
-      Wprintf("Cannot make (dis)association in this mode");
-   }
-   else if (schemobj->symschem != NULL || symbolobj->symschem != NULL) {
+   if (schemobj->symschem != NULL || symbolobj->symschem != NULL) {
       Wprintf("Both objects must be disassociated first.");
+#if TCL_WRAPPER
+      Tcl_SetResult(xcinterp, "Both objects must be disassociated first.", NULL);
+#endif
+      return False;
    }
    else {
       schemobj->symschem = symbolobj;
@@ -1005,7 +1083,7 @@ void schemassoc(objectptr schemobj, objectptr symbolobj)
       while (checkpagename(schemobj) < 0);
       setsymschem();	/* Set buttons and menu items appropriately */
    }
+   return True;
 }
 
-#endif
 /*-------------------------------------------------------------------------*/
