@@ -14,6 +14,44 @@ typedef unsigned int	u_int;
 typedef unsigned long	u_long;
 #endif
 
+#ifdef XC_WIN32
+#ifdef TCL_WRAPPER
+#include "tkwin32.h"
+#else
+#include "xcwin32.h"
+#endif
+#endif
+
+/*----------------------------------------------------------------------*/
+/* Graphics functions defined for X11 (i.e., not OpenGL)		*/
+/*----------------------------------------------------------------------*/
+
+#ifndef OPENGL
+
+#define DrawLine	XDrawLine
+#define DrawLines	XDrawLines
+#define DrawPoint	XDrawPoint
+#define DrawPoints	XDrawPoints
+#define FillPolygon	XFillPolygon
+
+#define SetForeground	XSetForeground
+#define SetBackground	XSetBackground
+#define SetFunction	XSetFunction
+#define SetThinLineAttributes	XSetLineAttributes
+#define SetLineAttributes(a, b, c, d, e, f) \
+	 XSetLineAttributes(a, b, ((c) >= 1.55 ? (int)(c + 0.45) : 0), d, e, f)
+#define SetDashes	XSetDashes
+#define SetFillStyle	XSetFillStyle
+#define SetStipple(a,b,c) XSetStipple(a,b,STIPPLE[c])
+#define flusharea()	
+
+#else
+
+#define SetThinLineAttributes	SetLineAttributes
+#define flusharea	glFlush
+
+#endif /* !OPENGL */
+
 /*----------------------------------------------------------------------*/
 /* Redefinition of fprintf() allows redirection of output to a console	*/
 /* in the Tcl interpreter-based version.				*/
@@ -27,15 +65,19 @@ typedef unsigned long	u_long;
   #define Flush fflush
 #endif
 
+#ifdef TCL_WRAPPER
+  #define malloc Tcl_Alloc
+  #define free(a) Tcl_Free((char *)(a))
+  #define realloc(a,b) Tcl_Realloc((char *)(a), b)
+  #undef strdup
+  #define strdup Tcl_Strdup
+  extern char *Tcl_Strdup(const char *);
+#endif
+
 /*----------------------------------------------------------------------*/
-/* Definition shortcuts 						*/
+/* Deal with 32/64 bit processors based on autoconf results.		*/
 /*----------------------------------------------------------------------*/
 
-#define XtnSetArg(a,b)  	XtSetArg(wargs[n], a, b); n++
-#define abs(a)			((a) < 0 ? -(a) : (a))
-#define sign(a)			((a) <= 0 ? -1 : 1)
-#define max(a,b)		((a) < (b) ? (b) : (a))
-#define min(a,b)		((a) < (b) ? (a) : (b))
 #if SIZEOF_VOID_P == SIZEOF_UNSIGNED_INT
 #define Number(a)		(void *)((u_int) a)
 typedef u_int pointertype;
@@ -46,6 +88,49 @@ typedef u_long pointertype;
 ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 #endif
 
+/*----------------------------------------------------------------------*/
+/* Basic element types 							*/
+/*									*/
+/* These values determine how a genericptr should be cast into one of	*/
+/* labelptr, polyptr, etc.  To query the element type, use the macros	*/
+/* ELEMENTTYPE(), IS_LABEL(), IS_POLYGON(), etc.  These macros mask the	*/
+/* bit field.  Note that an element marked with REMOVE_TAG will not be	*/
+/* recognized as any valid element, but otherwise, querying the type is	*/
+/* transparent to other bit settings (such as NETLIST_INVALID).		*/
+/*----------------------------------------------------------------------*/
+
+/* Single-bit flags */
+#define	OBJINST		0x01
+#define LABEL		0x02
+#define POLYGON		0x04
+#define ARC		0x08
+#define SPLINE  	0x10
+#define PATH		0x20
+#define GRAPHIC		0x40	/* reserved; unused, for now.		*/
+#define ARRAY		0x80	/* reserved; unused, for now.		*/
+
+#define REMOVE_TAG	0x100	/* element to be removed from netlist	*/
+#define NETLIST_INVALID 0x200	/* this element invalidates the netlist	*/
+#define SELECT_HIDE	0x400	/* this element cannot be selected	*/
+#define DRAW_HIDE	0x800	/* this element is not drawn.		*/
+
+/* Bit fields */
+#define ALL_TYPES 	(OBJINST | LABEL | POLYGON | ARC | SPLINE | PATH \
+				| GRAPHIC | ARRAY)
+#define VALID_TYPES 	(REMOVE_TAG | ALL_TYPES)
+
+/*----------------------------------------------------------------------*/
+/* Definition shortcuts 						*/
+/*----------------------------------------------------------------------*/
+
+#define XtnSetArg(a,b)  	XtSetArg(wargs[n], a, b); n++
+#define abs(a)			((a) < 0 ? -(a) : (a))
+#define sign(a)			((a) <= 0 ? -1 : 1)
+#ifndef min
+#define max(a,b)		((a) < (b) ? (b) : (a))
+#define min(a,b)		((a) < (b) ? (a) : (b))
+#endif
+
 /*----------------------------------------------------------------------*/
 /* Lengthier define constructs						*/
 /*----------------------------------------------------------------------*/
@@ -53,18 +138,30 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 /* Names used for convenience throughout the source code */
 
 #define topobject	areastruct.topinstance->thisobject
+#define hierobject	areastruct.hierstack->thisinst->thisobject
 
 #define PLIST_INCR(a) \
 	(a)->plist = (genericptr *) realloc ((a)->plist, \
 	((a)->parts + 1) * sizeof(genericptr))
-#define SELECTTYPE(a)   (*(topobject->plist + *(a)))->type
-#define SELTOCOLOR(a)	(*(topobject->plist + *(a)))->color
-#define GSELTOCOLOR(b, a) (*((b)->plist + *(a)))->color
+
 #define ENDPART		topobject->plist + topobject->parts
 #define EDITPART	topobject->plist + areastruct.editpart
 
-/* conversions from generic to specific types */
-/* specifically, from type (generic *) to type (polyptr), etc. */
+/* Type checks (argument "a" is type genericptr) */
+
+#define ELEMENTTYPE(a)	((a)->type & VALID_TYPES)
+
+#define IS_POLYGON(a)   (ELEMENTTYPE(a) == POLYGON)
+#define IS_LABEL(a)     (ELEMENTTYPE(a) == LABEL)
+#define IS_OBJINST(a)   (ELEMENTTYPE(a) == OBJINST)
+#define IS_ARC(a)       (ELEMENTTYPE(a) == ARC)
+#define IS_SPLINE(a)    (ELEMENTTYPE(a) == SPLINE)
+#define IS_PATH(a)      (ELEMENTTYPE(a) == PATH)
+#define IS_GRAPHIC(a)   (ELEMENTTYPE(a) == GRAPHIC)
+#define IS_ARRAY(a)     (ELEMENTTYPE(a) == ARRAY)
+
+/* Conversions from generic to specific types */
+/* specifically, from type (genericptr *) to type (polyptr), etc. */
 
 #define TOPOLY(a)	(*((polyptr *)(a)))
 #define TOLABEL(a)	(*((labelptr *)(a)))
@@ -72,16 +169,27 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 #define TOARC(a)	(*((arcptr *)(a)))
 #define TOSPLINE(a)	(*((splineptr *)(a))) 
 #define TOPATH(a)	(*((pathptr *)(a)))
+#define TOGRAPHIC(a)	(*((graphicptr *)(a)))
+#define TOGENERIC(a)	(*((genericptr *)(a)))
 
 /* conversions from a selection to a specific type */
 
-#define SELTOPOLY(a)	TOPOLY(topobject->plist + (*(a)))
-#define SELTOLABEL(a)	TOLABEL(topobject->plist + (*(a)))
-#define SELTOOBJINST(a)	TOOBJINST(topobject->plist + (*(a)))
-#define SELTOARC(a)	TOARC(topobject->plist + (*(a)))
-#define SELTOSPLINE(a)	TOSPLINE(topobject->plist + (*(a)))
-#define SELTOPATH(a)	TOPATH(topobject->plist + (*(a)))
-#define SELTOGENERIC(a) (*(topobject->plist + (*(a))))
+#define SELTOGENERICPTR(a) ((areastruct.hierstack == NULL) ? \
+			(topobject->plist + *(a)) : \
+			(hierobject->plist + *(a)))
+
+#define SELTOPOLY(a)	TOPOLY(SELTOGENERICPTR(a))
+#define SELTOLABEL(a)	TOLABEL(SELTOGENERICPTR(a))
+#define SELTOOBJINST(a)	TOOBJINST(SELTOGENERICPTR(a))
+#define SELTOARC(a)	TOARC(SELTOGENERICPTR(a))
+#define SELTOSPLINE(a)	TOSPLINE(SELTOGENERICPTR(a))
+#define SELTOPATH(a)	TOPATH(SELTOGENERICPTR(a))
+#define SELTOGRAPHIC(a)	TOGRAPHIC(SELTOGENERICPTR(a))
+#define SELTOGENERIC(a)	TOGENERIC(SELTOGENERICPTR(a))
+
+#define SELECTTYPE(a)   ((SELTOGENERIC(a))->type & ALL_TYPES)
+#define SELTOCOLOR(a)	((SELTOGENERIC(a))->color)
+
 
 /* creation of new elements */
 
@@ -99,7 +207,7 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 	PLIST_INCR(b); \
 	a = (objinstptr *)b->plist + b->parts; \
 	*a = (objinstptr) malloc(sizeof(objinst)); \
-	(*a)->type = OBJECT
+	(*a)->type = OBJINST
 #define NEW_ARC(a,b) \
 	PLIST_INCR(b); \
 	a = (arcptr *)b->plist + b->parts; \
@@ -115,12 +223,11 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 	a = (pathptr *)b->plist + b->parts; \
 	*a = (pathptr) malloc(sizeof(path)); \
 	(*a)->type = PATH
-
-/* selection from top-level object */
-
-#define objectselect(a)	 genobjectselect(a, topobject, areastruct.topinstance)
-#define easydraw(a, b)	 geneasydraw(a, b, topobject, areastruct.topinstance)
-#define drawselected(a)	 gendrawselected(a, topobject, areastruct.topinstance)
+#define NEW_GRAPHIC(a,b) \
+	PLIST_INCR(b); \
+	a = (graphicptr *)b->plist + b->parts; \
+	*a = (graphicptr) malloc(sizeof(graphic)); \
+	(*a)->type = GRAPHIC
 
 /*----------------------------------------------------------------------*/
 
@@ -151,10 +258,7 @@ typedef struct {
 #define MINAUTOSCALE 0.75 /* Won't automatically scale closer than this */
 #define MAXCHANGES 20 /* Number of changes to induce a temp file save	*/
 #define STIPPLES    8 /* Number of predefined stipple patterns		*/
-
-#ifdef SCHEMA
 #define PADSPACE   10 /* Spacing of pinlabels from their origins	*/
-#endif
 
 #ifdef HAVE_XPM
 #define TBBORDER   1  /* border around toolbar buttons */
@@ -178,8 +282,8 @@ typedef struct {
 
 /*----------------------------------------------------------------------*/
 
-#define RADFAC 0.0174532925199   /* pi / 180 */
-#define INVRFAC 57.295779       /* 180 / pi */
+#define RADFAC 0.0174532925199  /* (pi / 180) */
+#define INVRFAC 57.295779 /* (180 / pi) */
 
 /*----------------------------------------------------------------------*/
 
@@ -191,31 +295,47 @@ typedef struct {
 /* Event mode definitions (state of drawing area)			*/
 /*----------------------------------------------------------------------*/
 
-enum editmode {NORMAL_MODE = 0, 
-PENDING_MODE, PRESS_MODE, DELETE_MODE, COPY_MODE, COPY2_MODE, ROTATE_MODE,
-PUSH_MODE, EDIT_MODE, SELAREA_MODE, SELAREA2_MODE, DESEL_MODE, PAN_MODE,
-CATPAN_MODE, CATALOG_MODE, CATTEXT_MODE, FONTCAT_MODE, FONTCAT2_MODE,
-TEXT1_MODE, TEXT2_MODE, TEXT3_MODE, WIRE_MODE, POLY_MODE, BOX0_MODE,
-BOX_MODE, EBOX_MODE, ARC0_MODE, ARC_MODE, EARC_MODE, SPLINE0_MODE,
-SPLINE_MODE, ESPLINE_MODE, EPATH_MODE, LPARAM_MODE, IPARAM_MODE,
-ULPARAM_MODE, UIPARAM_MODE
-
-#ifdef SCHEMA
-, CONNECT_MODE, ASSOC_MODE
-#endif
-
+enum editmode {
+  NORMAL_MODE = 0,	/* On the drawing page, none of the situations below */
+  UNDO_MODE,		/* In the process of an undo/redo operation */
+  MOVE_MODE,		/* In the process of moving elements */
+  COPY_MODE,		/* In the process of copying elements */
+  PAN_MODE,		/* In the process of panning to follow the cursor */
+  SELAREA_MODE,		/* Area selection box */
+  RESCALE_MODE,		/* Interactive element rescaling box */
+  CATALOG_MODE,		/* On a library page, library directory, or page directory */
+  CATTEXT_MODE,		/* Editing an existing object name in the library */
+  FONTCAT_MODE,		/* Accessing the font character page from TEXT2_MODE */
+  EFONTCAT_MODE,	/* Accessing the font character page from TEXT3_MODE */
+  TEXT_MODE,		/* Creating a new label */
+  WIRE_MODE,		/* Creating a new polygon (wire) */
+  BOX_MODE,		/* Creating a new box */
+  ARC_MODE,		/* Creating a new arc */
+  SPLINE_MODE,		/* Creating a new spline */
+  ETEXT_MODE,		/* Editing an exiting label */
+  EPOLY_MODE,		/* Editing an existing polygon */
+  EARC_MODE,		/* Editing an existing arc */
+  ESPLINE_MODE,		/* Editing an existing spline */
+  EPATH_MODE,		/* Editing an existing path */
+  ASSOC_MODE		/* Choosing an associated schematic or symbol */
 };
 
 /*----------------------------------------------------------------------*/
 /* File loading modes							*/
 /*----------------------------------------------------------------------*/
 
-#define LOAD_MODES 5
-
-#define IMPORT     1
-#define PSBKGROUND 2
-#define SCRIPT	   3
-#define RECOVER    4
+enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
+#ifdef ASG
+	IMPORTSPICE,
+#endif
+#ifdef USE_WIN32_COM
+	COMSCRIPT,
+#ifdef USE_WIN32_DOTNET
+	COMDOTNET,
+#endif
+#endif
+	LOAD_MODES
+};
 
 /*----------------------------------------------------------------------*/
 /* Text justification styles and other parameters (bitmask)		*/
@@ -225,9 +345,14 @@ ULPARAM_MODE, UIPARAM_MODE
 #define RIGHT		2
 #define NOTBOTTOM	4
 #define TOP		8
-#define FLIPINV		16	/* 1 if text is flip-invariant */
-#define PINVISIBLE	32	/* 1 if pin visible outside of object */
-#define PINNOOFFSET	64	/* 0 if pin label offset from position */
+#define FLIPINV		16	/* 1 if text is flip-invariant		*/
+#define PINVISIBLE	32	/* 1 if pin visible outside of object	*/
+#define PINNOOFFSET	64	/* 0 if pin label offset from position	*/
+#define LATEXLABEL	128
+
+#define RLJUSTFIELD	3	/* right-left justification bit field	*/
+#define TBJUSTFIELD	12	/* top-bottom justification bit field	*/
+#define NONJUSTFIELD	240	/* everything but justification fields	*/
 
 /*----------------------------------------------------------------------*/
 /* Text string part: types						*/
@@ -250,8 +375,8 @@ ULPARAM_MODE, UIPARAM_MODE
 #define FONT_SCALE	14  /* font scale change; data = scale		*/
 #define FONT_COLOR	15  /* font color change; data = color		*/
 #define KERN		16  /* set new kern values; data = kern x, y	*/
-#define PARAM_START	17  /* bounds a parameter; data = param #	*/
-#define PARAM_END	18  /* bounds a parameter; data = param #	*/
+#define PARAM_START	17  /* bounds a parameter; data = param key	*/
+#define PARAM_END	18  /* bounds a parameter; no data 		*/
 
 #define SPECIAL		63  /* used only when called from menu		*/
 #define NULL_TYPE	255 /* used as a placeholder			*/
@@ -270,6 +395,7 @@ ULPARAM_MODE, UIPARAM_MODE
 #define DEC_INCH	0
 #define FRAC_INCH	1
 #define CM		2
+#define INTERNAL	3
 
 /*----------------------------------------------------------------------*/
 /* Library types							*/
@@ -295,8 +421,12 @@ ULPARAM_MODE, UIPARAM_MODE
 #define STIP1 64
 #define STIP2 128
 #define FILLSOLID 224  /* = 32 + 64 + 128 */
+#ifdef OPAQUE
+#undef OPAQUE
+#endif
 #define OPAQUE 256
 #define BBOX 512
+#define SQUARECAP 1024
 
 /*----------------------------------------------------------------------*/
 /* Box edit styles							*/
@@ -331,12 +461,11 @@ ULPARAM_MODE, UIPARAM_MODE
 /* Schematic object types and pin label types	   			*/
 /*----------------------------------------------------------------------*/
 
-#ifdef SCHEMA
-
-#define SCHEMATIC 0		/* Schematic on page */
-#define TRIVIAL 1		/* Symbol as non-schematic element */
-#define SYMBOL 2		/* Symbol associated with a schematic */
-#define FUNDAMENTAL 3		/* Standalone symbol */
+#define PRIMARY 0		/* Primary (master) schematic page */
+#define SECONDARY 1		/* Secondary (slave) schematic page */
+#define TRIVIAL 2		/* Symbol as non-schematic element */
+#define SYMBOL 3		/* Symbol associated with a schematic */
+#define FUNDAMENTAL 4		/* Standalone symbol */
 
 #define LOCAL 1
 #define GLOBAL 2
@@ -344,8 +473,6 @@ ULPARAM_MODE, UIPARAM_MODE
 
 #define HIERARCHY_LIMIT 256	/* Stop if recursion goes this deep */
 
-#endif
-
 /*----------------------------------------------------------------------*/
 /* Save types.	This list incorporates "ALL_PAGES", below.		*/
 /*----------------------------------------------------------------------*/
@@ -373,7 +500,7 @@ ULPARAM_MODE, UIPARAM_MODE
 /* Cursor definitions 							*/
 /*----------------------------------------------------------------------*/
 
-#define NUM_CURSORS	10
+#define NUM_CURSORS	11
 
 #define ARROW 		appcursors[0]
 #define CROSS 		appcursors[1]
@@ -385,6 +512,9 @@ ULPARAM_MODE, UIPARAM_MODE
 #define CIRCLE		appcursors[7]
 #define QUESTION	appcursors[8]
 #define WAITFOR		appcursors[9]
+#define HAND		appcursors[10]
+
+#define DEFAULTCURSOR	(*areastruct.defaultcursor)
 
 /*----------------------------------------------------------------------*/
 /* selection-mechanism structures					*/
@@ -393,23 +523,11 @@ ULPARAM_MODE, UIPARAM_MODE
 typedef XPoint* pointlist;
 typedef XfPoint* fpointlist;
 
-/*----------------------------------------------------------------------*/
-/* select types 							*/
-/*----------------------------------------------------------------------*/
-
-#define	OBJECT	1
-#define LABEL	2
-#define POLYGON	4
-#define ARC	8
-#define SPLINE  16
-#define PATH	32
-#define SEL_ANY	63  /* for selection mechanism only */
-
 /*----------------------------------------------------------------------*/
 /* Allowed parameterization types					*/
 /*----------------------------------------------------------------------*/
 
-#define P_POSITION	0	/* mode only, not a real parameter */
+#define P_NUMERIC	0	/* uncommitted numeric parameter */
 #define P_SUBSTRING	1
 #define P_POSITION_X	2
 #define P_POSITION_Y	3
@@ -423,6 +541,8 @@ typedef XfPoint* fpointlist;
 #define P_SCALE		11
 #define P_LINEWIDTH	12
 #define P_COLOR		13
+#define P_EXPRESSION	14
+#define P_POSITION	15	/* mode only, not a real parameter */
 
 /*----------------------------------------------------------------------*/
 /* Labels are constructed of strings and executables 			*/
@@ -437,7 +557,6 @@ typedef struct _stringpart {
      u_char	*string;
      int	color;
      int	font;
-     int	paramno;
      float	scale;
      short	kern[2];
    }		data;
@@ -452,32 +571,50 @@ typedef struct _stringpart {
 /* (Add types as necessary)						*/
 /*----------------------------------------------------------------------*/
 
-enum paramtypes {XC_INT = 0, XC_FLOAT, XC_STRING};
+enum paramtypes {XC_INT = 0, XC_FLOAT, XC_STRING, XC_EXPR};
 
-typedef struct {
+/* Object parameters: general key:value parameter model */
+/* Note that this really should be a hash table, not a linked list. . . */
+
+typedef struct _oparam *oparamptr;
+
+typedef struct _oparam {
+   char *	key;		/* name of the parameter */
    u_char	type;		/* type is from paramtypes list above */
    u_char	which;		/* what the parameter represents (P_*) */
    union {
-      stringpart *string;
+      stringpart *string;	/* xcircuit label type */
+      char	 *expr;		/* TCL (string) expression */
       int	  ivalue;	/* also covers type short int by typecasting */
       float	  fvalue;
    } parameter;			/* default or substitution value */
-} oparam, *oparamptr;
+   oparamptr	next;		/* next parameter in linked list */
+} oparam;
 
-typedef struct {
-   u_char	paramno;	/* parameter number to get value from.	   */
-   u_char	pointno;	/* point number in point array, for polygons */
-} eparam, *eparamptr;
+/* Element parameters: reference back to the object's parameters */
+/* These parameters are forward-substituted when descending into */
+/* an object instance.						 */
+/* Note that this really should be a hash table, not a linked list. . . */
+
+typedef struct _eparam *eparamptr;
+
+typedef struct _eparam {
+   char *	key;	    /* name of the parameter */
+   union {
+      int	pointno;    /* point number in point array, for polygons */
+      char	*refkey;    /* parameter reference key, for instances */
+   } pdata;
+   eparamptr	next;	    /* next parameter in linked list */
+} eparam;
 
 /*----------------------------------------------------------------------*/
 /* Generic element type	is a superset of all elements.			*/	
 /*----------------------------------------------------------------------*/
 
 typedef struct {
-   u_char	type;		/* type is LABEL, POLYGON, etc., from below */
+   u_short	type;		/* type is LABEL, POLYGON, etc., from below */
    int		color;
    eparamptr	passed;
-   u_char	num_params;
 } generic, *genericptr;		/* (convenience function for retypecasting) */
 
 /*----------------------------------------------------------------------*/
@@ -496,19 +633,16 @@ typedef struct {
 typedef struct _xcobject *objectptr;
 
 typedef struct {
-   u_char	type;
+   u_short	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
-   u_char	num_params;
    XPoint	position;
    short	rotation;
    float	scale;
    objectptr	thisobject;
-   oparamptr    *params;	/* parameter substitutions for this instance */
+   oparamptr	params;		/* parameter substitutions for this instance */
    BBox		bbox;		/* per-instance bounding box information */
-#ifdef SCHEMA
    BBox		*schembbox;	/* Extra bounding box for pin labels */
-#endif
 } objinst, *objinstptr;
 
 /* Linked-list for objects */
@@ -516,7 +650,8 @@ typedef struct {
 typedef struct _objlist *objlistptr;
 
 typedef struct _objlist {
-   objectptr	thisobject;
+   int		libno;		/* library in which object appears */
+   objectptr	thisobject;	/* pointer to the object */
    objlistptr	next;
 } objlist;
 
@@ -541,22 +676,36 @@ typedef struct _liblist {
    liblistptr	next;
 } liblist;
 
+/*----------------------------------------------------------------------*/
+/* Generalized graphic object (Tcl/Tk only)				*/
+/*----------------------------------------------------------------------*/
+
+typedef struct {
+   u_short	  type;
+   int		  color;	/* foreground, for bitmaps only */
+   eparamptr	  passed;	/* numerical parameters passed from above */
+   XPoint	  position;
+   short	  rotation;
+   float	  scale;
+   XImage	  *source;	/* source data */
+   XImage	  *target;	/* target (scaled) data */
+   Pixmap	  clipmask;	/* clipmask for non-manhattan rotations */
+   Boolean	  valid;	/* does target need to be regenerated? */
+} graphic, *graphicptr;
+
 /*----------------------------------------------------------------------*/
 /* Label								*/
 /*----------------------------------------------------------------------*/
 
 typedef struct {
-   u_char	type;
+   u_short	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
-   u_char	num_params;
    XPoint	position;
    short	rotation;
    float	scale;
    short	justify;
-#ifdef SCHEMA
    u_char	pin;
-#endif
    stringpart	*string;
 } label, *labelptr;
 
@@ -565,10 +714,9 @@ typedef struct {
 /*----------------------------------------------------------------------*/
 
 typedef struct {
-   u_char	type;
+   u_short	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
-   u_char	num_params;
    u_short	style;
    float	width;
    short	number;
@@ -580,10 +728,9 @@ typedef struct {
 /*----------------------------------------------------------------------*/
 
 typedef struct {
-   u_char	type;
+   u_short	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
-   u_char	num_params;
    u_short	style;
    float	width;
    XPoint	ctrl[4];
@@ -596,10 +743,9 @@ typedef struct {
 /*----------------------------------------------------------------------*/
 
 typedef struct {
-   u_char	type;
+   u_short	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
-   u_char	num_params;
    u_short	style;
    float	width;
    short	radius; 	/* x-axis radius */
@@ -617,29 +763,76 @@ typedef struct {
 /*----------------------------------------------------------------------*/
 
 typedef struct {
-   u_char	type;
+   u_short	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
-   u_char	num_params;
    u_short	style;
    float	width;
    short	parts;
    genericptr	*plist;		/* to be retypecast to polygon, arc, or spline */
 } path, *pathptr;
 
+/*----------------------------------------------------------------------*/
+/* selection from top-level object 					*/
+/*----------------------------------------------------------------------*/
+
+/*----------------------------------------------------------------------*/
+/* Undo mechanism definitions						*/
+/*----------------------------------------------------------------------*/
+
+enum UNDO_MODES {UNDO_DONE = 0, UNDO_MORE, MODE_CONNECT, MODE_RECURSE_WIDE,
+	MODE_RECURSE_NARROW};
+
+typedef struct _selection *selectionptr;
+
+typedef struct _selection {
+   int selects;
+   short *selectlist;
+   objinstptr thisinst;
+   selectionptr next;
+} selection;
+
+#define select_element(a) recurse_select_element(a, UNDO_MORE) 
+#define select_add_element(a) recurse_select_element(a, UNDO_DONE) 
+
+#define easydraw(a, b)	 geneasydraw(a, b, topobject, areastruct.topinstance)
+
 /*----------------------------------------------------------------------*/
 /* Netlist structures for schematic capture				*/
 /*----------------------------------------------------------------------*/
 
-#ifdef SCHEMA
+/* Structure to hold net and subnet IDs for a bus */
+
+typedef struct {
+   int netid;
+   int subnetid;
+} buslist;
+
+/* Structure mimicking the top part of a Polylist or Labellist	*/
+/* when we just want the netlist information and don't care	*/
+/* which one we're looking at.					*/
+
+typedef struct {
+   union {
+      int id;
+      buslist *list;
+   } net;
+   int subnets;
+} Genericlist; 
 
 /* Linked polygon list */
 
 typedef struct _Polylist *PolylistPtr;
 typedef struct _Polylist
 {
+   union {
+      int  id;		/* A single net ID, if subnets == 0	*/
+      buslist *list;	/* List of net and subnet IDs for a bus	*/
+   } net;
+   int subnets;		/* Number of subnets; 0 if no subnets	*/
+   objectptr cschem;	/* Schematic containing the polygon	*/
    polyptr poly;
-   PolylistPtr next;
+   PolylistPtr next;	/* Next polygon in the linked list	*/
 } Polylist;
 
 /* Linked label list */
@@ -647,21 +840,26 @@ typedef struct _Polylist
 typedef struct _Labellist *LabellistPtr;
 typedef struct _Labellist
 {
+   union {
+      int  id;		/* A single net ID, if subnets == 0	*/
+      buslist *list;	/* List of net and subnet IDs for a bus	*/
+   } net;
+   int subnets;		/* Number of subnets; 0 if no subnets	*/
+   objectptr cschem;	/* Schematic containing the label	*/
+   objinstptr cinst;	/* Specific label instance, if applicable */
    labelptr label;
    LabellistPtr next;
 } Labellist;
 
-/* List of object's networks */
+/* List of object's networks by (flattened) name */
 
-typedef struct _Netlist *NetlistPtr;
-typedef struct _Netlist
+typedef struct _Netname *NetnamePtr;
+typedef struct _Netname
 {
    int netid;
-   PolylistPtr polygons;
-   LabellistPtr labels;
    stringpart *localpin;
-   NetlistPtr next;
-} Netlist;
+   NetnamePtr next;
+} Netname;
 
 /* List of object's I/O ports */
 
@@ -679,9 +877,11 @@ typedef struct _Portlist
 typedef struct _Calllist *CalllistPtr;
 typedef struct _Calllist
 {
-   objectptr   callobj;
-   objinstptr  callinst;
-   int         devindex;	/* if non-zero, index of device in netlist */
+   objectptr   cschem;		/* Schematic containing the instance called */
+   objinstptr  callinst;	/* Instance called */
+   objectptr   callobj;		/* Object of instance called */
+   char	       *devname;	/* if non-null, name of device in netlist */
+   int         devindex;	/* if non-negative, index of device in netlist */
    PortlistPtr ports;
    CalllistPtr next;
 } Calllist;
@@ -700,6 +900,7 @@ struct Pstr {
 };
 
 struct Ptab {
+   objectptr cschem;
    struct Pnet *nets;
    struct Pstr *pins;
    struct Ptab *next;
@@ -710,12 +911,10 @@ struct Ptab {
 /*----------------------------------------------------------------------*/
 
 typedef struct {
-   int		netid;
+   Genericlist	*netlist;
    objinstptr	thisinst;
 } Highlight;
 
-#endif
-
 /*----------------------------------------------------------------------*/
 /* Main object structure						*/
 /*----------------------------------------------------------------------*/
@@ -730,21 +929,19 @@ typedef struct _xcobject {
 				/* parameterized elements) */
    short	parts;
    genericptr	*plist;	 	/* to be retypecast to label, polygon, etc. */
-   u_char	num_params;	/* number of calling parameters */
-   oparamptr	*params;	/* list of parameter defaults */
+   oparamptr	params;		/* list of parameters, with default values */
    
-#ifdef SCHEMA
    Highlight	highlight;	/* net to be highlighted on redraw */
    u_char	schemtype;
    objectptr	symschem;	/* schematic page support */
-
-   char		*devname;	/* if non-null, name of device in netlist */
    Boolean	valid;		/* Is current netlist valid? */
    Boolean	traversed;	/* Flag to indicate object was processed */
-   NetlistPtr   netlist;
-   PortlistPtr  portlist;
-   CalllistPtr  calllist;
-#endif
+   LabellistPtr	labels;		/* Netlist pins	 */
+   PolylistPtr	polygons;	/* Netlist wires */
+   PortlistPtr  ports;		/* Netlist ports */
+   CalllistPtr  calls;		/* Netlist subcircuits and connections */
+   NetnamePtr   netnames;	/* Local names for flattening */
+				/* (this probably shouldn't be here. . .) */
 } object;
 
 /*----------------------------------------------------------------------*/
@@ -832,6 +1029,9 @@ typedef struct _alias {
 #define xcRemoveTimeOut			XtRemoveTimeOut
 #define xcTimeOutProc			XtTimerCallbackProc
 #define xcIntervalId			XtIntervalId
+#ifndef ClientData
+   #define ClientData			XtPointer
+#endif
 
 #endif
 
@@ -877,10 +1077,8 @@ enum {DIRECTORY = 0, MATCH, NONMATCH};	/* file types */
 /*----------------------------------------------------------------------*/
 
 typedef struct {
-#ifdef SCHEMA
    /* schematic layout colors */
-   Pixel	globalcolor, localcolor, infocolor;
-#endif
+   Pixel	globalcolor, localcolor, infocolor, ratsnestcolor;
 
    /* non-schematic layout color(s) */
    Pixel	bboxpix;
@@ -899,21 +1097,26 @@ typedef struct {
    Pixel	auxpix2, barpix2, parampix2;
 
    int		width, height, timeout;
-   XFontStruct	*xcfont, *helpfont, *filefont, *textfont, *titlefont;
+   XFontStruct	*helpfont, *filefont;
+
+#ifndef TCL_WRAPPER
+   XFontStruct	*xcfont, *textfont, *titlefont;
+#endif
+
 } ApplicationData, *ApplicationDataPtr;
 
 /*----------------------------------------------------------------------*/
 /* Macros for GC color and function handling				*/
 /*----------------------------------------------------------------------*/
 
-#define XTopSetForeground(a) if (a == DEFAULTCOLOR) XSetForeground(dpy, \
-	areastruct.gc, FOREGROUND); else XSetForeground(dpy, areastruct.gc, a)
+#define XTopSetForeground(a) if (a == DEFAULTCOLOR) SetForeground(dpy, \
+	areastruct.gc, FOREGROUND); else SetForeground(dpy, areastruct.gc, a)
 
-#define XSetXORFg(a,b) if (a == DEFAULTCOLOR) XSetForeground(dpy, \
-	areastruct.gc, FOREGROUND ^ b); else XSetForeground(dpy, areastruct.gc,\
+#define XSetXORFg(a,b) if (a == DEFAULTCOLOR) SetForeground(dpy, \
+	areastruct.gc, FOREGROUND ^ b); else SetForeground(dpy, areastruct.gc,\
 	a ^ b)
 
-#define XcSetFunction(z) XSetFunction(dpy, areastruct.gc, z); \
+#define XcSetFunction(z) SetFunction(dpy, areastruct.gc, z); \
 	areastruct.gctype = z
 
 #define XcSetForeground(z) XTopSetForeground(z); areastruct.gccolor = \
@@ -975,35 +1178,67 @@ typedef struct _keybinding {
 
 /*----------------------------------------------------------------------*/
 /* Enumeration of functions available for binding to keys/buttons	*/
+/* IMPORTANT!  Do not alter this list without also fixing the text	*/
+/* in keybindings.c!							*/
 /*----------------------------------------------------------------------*/
 
 enum {
-   XCF_ENDDATA = -2, XCF_SPACER,
-   XCF_Page, XCF_Justify, XCF_Superscript, XCF_Subscript, XCF_Normalscript,
-   XCF_Nextfont, XCF_Boldfont, XCF_Italicfont, XCF_Normalfont, XCF_Underline,
-   XCF_Overline, XCF_ISO_Encoding, XCF_Return, XCF_Halfspace, XCF_Quarterspace,
-   XCF_Special, XCF_TabStop, XCF_TabForward, XCF_TabBackward,
-   XCF_Text_Return, XCF_Text_Delete, XCF_Text_Right, XCF_Text_Left,
-   XCF_Text_Up, XCF_Text_Down, XCF_Text_Split,
-   XCF_Text_Home, XCF_Text_End, XCF_Return_Char, XCF_Parameter,
-   XCF_Edit_Param, XCF_Edit_Break, XCF_Edit_Delete, XCF_Edit_Insert,
-   XCF_Edit_Next, XCF_Attach, XCF_Next_Library, XCF_Library_Directory,
-   XCF_Library_Move, XCF_Library_Copy, XCF_Library_Edit, XCF_Library_Delete,
-   XCF_Library_Duplicate, XCF_Library_Hide, XCF_Library_Virtual,
-   XCF_Page_Directory, XCF_Library_Pop,
-   XCF_Help, XCF_Redraw, XCF_View, XCF_Zoom_In, XCF_Zoom_Out, XCF_Pan,
-   XCF_Double_Snap, XCF_Halve_Snap, XCF_Pan_Left, XCF_Pan_Right, XCF_Pan_Up,
-   XCF_Pan_Down, XCF_Write, XCF_Rotate, XCF_Flip_X, XCF_Flip_Y, XCF_Snap,
-   XCF_SnapTo, XCF_Pop, XCF_Push, XCF_Delete, XCF_Select, XCF_Box, XCF_Arc,
-   XCF_Text, XCF_Exchange, XCF_Copy, XCF_Join, XCF_Unjoin, XCF_Spline,
-   XCF_Edit, XCF_Undelete, XCF_Select_Save, XCF_Unselect, XCF_Dashed,
-   XCF_Dotted, XCF_Solid, XCF_Prompt, XCF_Dot, XCF_Wire, XCF_Start,
-   XCF_Finish, XCF_Cancel, XCF_Nothing, XCF_Exit,
-
-#ifdef SCHEMA
-   XCF_Netlist, XCF_Swap, XCF_Pin_Label, XCF_Pin_Global, XCF_Info_Label,
-   XCF_Connectivity, XCF_Sim, XCF_SPICE, XCF_PCB, XCF_SPICEflat,
-#endif
+   XCF_ENDDATA = -2,		XCF_SPACER  /* -1 */,
+   XCF_Page	 /* 0 */,	XCF_Justify /* 1 */,
+   XCF_Superscript /* 2 */,	XCF_Subscript /* 3 */,
+   XCF_Normalscript /* 4 */,	XCF_Nextfont /* 5 */,
+   XCF_Boldfont /* 6 */,	XCF_Italicfont /* 7 */,
+   XCF_Normalfont /* 8 */,	XCF_Underline /* 9 */,
+   XCF_Overline /* 10 */,	XCF_ISO_Encoding /* 11 */,
+   XCF_Halfspace /* 12 */,	XCF_Quarterspace /* 13 */,
+   XCF_Special /* 14 */,	XCF_TabStop /* 15 */,
+   XCF_TabForward /* 16 */,	XCF_TabBackward /* 17 */,
+   XCF_Text_Return /* 18 */,	XCF_Text_Delete /* 19 */,
+   XCF_Text_Right /* 20 */,	XCF_Text_Left /* 21 */,
+   XCF_Text_Up /* 22 */,	XCF_Text_Down /* 23 */,
+   XCF_Text_Split /* 24 */,	XCF_Text_Home /* 25 */,
+   XCF_Text_End /* 26 */,	XCF_Linebreak /* 27 */,
+   XCF_Parameter /* 28 */,	XCF_Edit_Param /* 29 */,
+   XCF_Edit_Break /* 30 */,	XCF_Edit_Delete /* 31 */,
+   XCF_Edit_Insert /* 32 */,	XCF_Edit_Append /* 33 */,
+   XCF_Edit_Next /* 34 */,	XCF_Attach /* 35 */,
+   XCF_Next_Library /* 36 */,	XCF_Library_Directory /* 37 */,
+   XCF_Library_Move /* 38 */,	XCF_Library_Copy /* 39 */,
+   XCF_Library_Edit /* 40 */,	XCF_Library_Delete /* 41 */,
+   XCF_Library_Duplicate /* 42 */, XCF_Library_Hide /* 43 */,
+   XCF_Library_Virtual /* 44 */, XCF_Page_Directory /* 45 */,
+   XCF_Library_Pop /* 46 */,	XCF_Virtual /* 47 */,
+   XCF_Help /* 48 */,		XCF_Redraw /* 49 */,
+   XCF_View /* 50 */,		XCF_Zoom_In /* 51 */,
+   XCF_Zoom_Out /* 52 */,	XCF_Pan /* 53 */,
+   XCF_Double_Snap /* 54 */,	XCF_Halve_Snap /* 55 */,
+   XCF_Write /* 56 */,		XCF_Rotate /* 57 */,
+   XCF_Flip_X /* 58 */,		XCF_Flip_Y /* 59 */,
+   XCF_Snap /* 60 */,		XCF_SnapTo /* 61 */,
+   XCF_Pop /* 62 */,		XCF_Push /* 63 */,
+   XCF_Delete /* 64 */,		XCF_Select /* 65 */,
+   XCF_Box /* 66 */,		XCF_Arc /* 67 */,
+   XCF_Text /* 68 */,		XCF_Exchange /* 69 */,
+   XCF_Copy /* 70 */,		XCF_Move /* 71 */,
+   XCF_Join /* 72 */,		XCF_Unjoin /* 73 */,
+   XCF_Spline /* 74 */,		XCF_Edit /* 75 */,
+   XCF_Undo /* 76 */,		XCF_Redo /* 77 */,
+   XCF_Select_Save /* 78 */,	XCF_Unselect /* 79 */,
+   XCF_Dashed /* 80 */,		XCF_Dotted /* 81 */,
+   XCF_Solid /* 82 */,		XCF_Prompt /* 83 */,
+   XCF_Dot /* 84 */,		XCF_Wire /* 85 */,
+   XCF_Cancel /* 86 */,		XCF_Nothing /* 87 */,
+   XCF_Exit /* 88 */,		XCF_Netlist /* 89 */,
+   XCF_Swap /* 90 */,		XCF_Pin_Label /* 91 */,
+   XCF_Pin_Global /* 92 */,	XCF_Info_Label /* 93 */,
+   XCF_Graphic	/* 94 */,	XCF_Rescale /* 95 */,
+   XCF_SelectBox /* 96 */,	XCF_Connectivity /* 97 */,
+   XCF_Continue_Copy /* 98 */,	XCF_Continue_Element /* 99 */,
+   XCF_Finish_Copy /* 100 */,	XCF_Finish_Element /* 101 */,
+   XCF_Finish /* 102 */,	XCF_Cancel_Last /* 103 */,
+   XCF_Sim /* 104 */,		XCF_SPICE /* 105 */,
+   XCF_PCB /* 106 */,		XCF_SPICEflat /* 107 */,
+   XCF_Text_Delete_Param /* 108 */,
    NUM_FUNCTIONS
 };
 
@@ -1015,6 +1250,7 @@ typedef struct {
    /* per-drawing-page parameters */
    objinstptr	pageinst;
    char		*filename;	/* file to save as */
+   u_char	idx;		/* page index */
    psbkground	background;	/* background rendered file info */
    float	wirewidth;
    float	outscale;
@@ -1027,6 +1263,17 @@ typedef struct {
    XPoint	pagesize;	/* size of page to print on */
 } Pagedata;
 
+/*----------------------------------------------------------------------*/
+/* Structure holding information about graphic images used.  These hold	*/
+/* the original data for the images, and may be shared.			*/
+/*----------------------------------------------------------------------*/
+
+typedef struct {
+   XImage	*image;
+   int		refcount;
+   char		*filename;
+} Imagedata;
+
 /*----------------------------------------------------------------------*/
 /* The main globally-accessible data structure.  This structure holds	*/
 /* all the critical data needed by the drawing window 			*/
@@ -1067,11 +1314,10 @@ typedef struct {
    Boolean	invert;
    Boolean	mapped;		/* indicates if window is drawable */
    Boolean	filefilter;	/* Is the file list filtered? */
+   char		buschar;	/* Character indicating vector notation */
    Boolean	editinplace;
-#ifdef SCHEMA
    Boolean	schemon;
    Boolean	pinpointon;
-#endif
 #ifdef HAVE_XPM
    Boolean	toolbar_on;
 #endif
@@ -1080,18 +1326,33 @@ typedef struct {
    XPoint	save, origin;
    short	selects;
    short	*selectlist;
-   short	lastselects;
    short	lastlibrary;
-   short	*selectlast;
    objinstptr	topinstance;
    objectptr	editstack;
    Matrixptr	MatStack;
    pushlistptr	stack;
+   pushlistptr	hierstack;
    short	editcycle;
    short	editpart, editsubpart;
    char		*lastbackground;
+   Cursor	*defaultcursor;
 } Clientdata;
 
+/* Record for undo function */ 
+
+typedef struct _undostack *Undoptr;
+
+typedef struct _undostack {
+   Undoptr next;	/* next record in undo stack */
+   Undoptr last;	/* double-linked for "redo" function */
+   u_int type;		/* type of event */
+   short idx;		/* counter for undo event */
+   objinstptr thisinst;	/* instance of object in which event occurred */
+   int idata;		/* simple undedicated integer datum */
+   char *undodata;	/* free space to be malloc'd */
+			/* (size dependent on "type") */
+} Undostack;
+
 /* This whole thing needs to be cleaned up. . . now I've got objects in */
 /* "library" which are repeated both in the "instlist" pair and the	*/
 /* "thisobject" pointer in each instance.  The instance is repeated on	*/
@@ -1099,9 +1360,10 @@ typedef struct {
 /* the library page, and not be destroyed on every call to "composelib"	*/
 
 typedef struct {
+   char		*filename;	/* Last file loaded into the library */
    short	number;
    objectptr	*library;
-   liblistptr	instlist;  /* List of instances */
+   liblistptr	instlist;	/* List of instances */
 } Library;
 
 /*----------------------------------------------------------------------*/
@@ -1111,16 +1373,21 @@ typedef struct {
 typedef struct {
    char		*tempfile;
    char		*tempdir;
+   Boolean	retain_backup;
    xcIntervalId	timeout_id;
    int		save_interval;
+   short	do_replace;
    u_short	new_changes;
-   short	pages;
    short	numlibs;
+   short	pages;
    Pagedata	**pagelist;
-   Library	delbuffer;
+   Undoptr	undostack;	/* actions to undo */
+   Undoptr	redostack;	/* actions to redo */
    Library	fontlib;
    Library	*userlibs;
    objinstptr   *libtop;
+   Imagedata	*imagelist;
+   short	images;
 } Globaldata;
 
 /*----------------------------------------------------------------------*/
@@ -1146,6 +1413,8 @@ typedef struct _menustruct {
 /* Structure for calling routines from the Toolbar icons		*/
 /*----------------------------------------------------------------------*/
 
+typedef struct _toolbarstruct *toolbarptr;
+
 typedef struct _toolbarstruct {
 	char *name;
 	char **icon_data;
@@ -1158,3 +1427,4 @@ typedef struct _toolbarstruct {
 /* Menus and Toolbars are taken care of entirely by scripts in the Tcl/	*/
 /* Tk version of xcircuit.						*/
 /*----------------------------------------------------------------------*/
+
